package com.starbuds.server.service.data.config;

import com.aliyuncs.profile.DefaultProfile;
import com.aliyuncs.profile.IClientProfile;
import com.starbuds.server.common.pojo.message.AliyunConfig;
import com.starbuds.server.common.service.core.ConfigService;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.List;

/**
 * 阿里云鉴黄配置
 *
 * @author aoe
 * @date 2020/5/28
 */
@Configuration
public class GreenConfig {

    @Reference(version = "1.0.0")
    private ConfigService configService;

    @Value("${aliyun.green.regionId}")
    private String greenRegionId;

    @Value("${aliyun.green.scenes}")
    private String greenScenes;

    /**
     * 内容安全配置
     */
    private IClientProfile greenProfile;

    /**
     * 检测内容类型
     */
    private List<String> greenScenesList;

    @PostConstruct
    private void init() {
        AliyunConfig aliyunAccessConfig = configService.getAliyunConfig();

        // 内容安全
        greenProfile = DefaultProfile
                .getProfile(greenRegionId, aliyunAccessConfig.getAccessKey(), aliyunAccessConfig.getAccessSecret());
        String[] scenesArr = greenScenes.split(",");
        greenScenesList = new ArrayList<>(scenesArr.length);
        for (String s : scenesArr) {
            greenScenesList.add(s);
        }
    }

    public IClientProfile getGreenProfile() {
        return greenProfile;
    }

    public List<String> getGreenScenesList() {
        return greenScenesList;
    }
}
