package com.starbuds.server.service.data.dao;

import com.starbuds.server.common.pojo.data.MusicVO;
import com.starbuds.server.common.pojo.db.MusicDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/8/3 16:34
 * @Description 音乐持久层
 */
@Mapper
@Repository
public interface MusicDAO {

    /**
     * 插入
     *
     * @param musicDO
     * @return
     */
    int insert(MusicDO musicDO);

    /**
     * 获取上传列表
     *
     * @param userId
     * @param lastId
     * @param pageSize
     * @return
     */
    List<MusicVO> selectUploadList(@Param("userId") Long userId,
                                   @Param("lastId") Long lastId,
                                   @Param("pageSize") Integer pageSize);

    /**
     * 获取用户id
     *
     * @param musicId
     * @return
     */
    Long selectUserId(@Param("musicId") Long musicId);

    /**
     * 判断音乐是否存在
     *
     * @param musicId
     * @return
     */
    int countById(@Param("musicId") Long musicId);

    /**
     * 更新状态
     *
     * @param musicId
     * @param status
     * @param updateTime
     * @return
     */
    int updateStatus(@Param("musicId") Long musicId,
                     @Param("status") int status,
                     @Param("updateTime") Long updateTime);

    /**
     * 判断音乐是否可下载
     *
     * @param musicId
     * @param status
     * @param shelfStatus
     * @return
     */
    int countByIdAndStatus(@Param("musicId") Long musicId,
                           @Param("status") Integer status,
                           @Param("shelfStatus") Integer shelfStatus);

    /**
     * 获取音乐url
     *
     * @param musicId
     * @return
     */
    String selectMusicUrl(@Param("musicId") Long musicId);

    /**
     * 递增下载次数
     *
     * @param musicId
     * @param updateTime
     * @return
     */
    int increaseDownloadCount(@Param("musicId") Long musicId, @Param("updateTime") Long updateTime);

    /**
     * 根据id查询
     *
     * @param musicId
     * @return
     */
    MusicDO selectById(@Param("musicId") Long musicId);

    /**
     * 根据id集合查询
     *
     * @param musicIds
     * @return
     */
    List<MusicVO> selectByIds(@Param("musicIds") List<Long> musicIds);

    /**
     * 获取下载次数
     *
     * @param musicId
     * @return
     */
    Integer selectDownloadCount(@Param("musicId") Long musicId);

    /**
     * 获取音乐状态
     *
     * @param musicId
     * @return
     */
    Integer selectMusicStatus(@Param("musicId") Long musicId);

    /**
     * 获取用户id
     *
     * @param musicIdList
     * @return
     */
    List<Long> selectUserIds(@Param("musicIdList") List<Long> musicIdList);

    /**
     * 获取上传数量
     *
     * @param userId
     * @return
     */
    int selectUploadCount(@Param("userId") Long userId);
}
