package com.starbuds.server.service.data.es.entity;

import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.data.annotation.Id;
import org.springframework.data.elasticsearch.annotations.Document;
import org.springframework.data.elasticsearch.annotations.Field;
import org.springframework.data.elasticsearch.annotations.FieldType;
import org.springframework.data.elasticsearch.annotations.GeoPointField;
import org.springframework.data.elasticsearch.core.geo.GeoPoint;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/27 19:46
 * @Description 用户ES实体
 */
@NoArgsConstructor
@Data
@Document(indexName = "user", shards = 1, replicas = 0, refreshInterval = "-1")
public class UserEntity {

    /**
     * 主键
     */
    @Id
    private Long userId;

    /**
     * 用户姓名
     */
    @Field(type = FieldType.Keyword)
    private String userName;

    /**
     * 用户姓名IK分词器
     */
    @Field(type = FieldType.Text)
    private String userNameIK;

    /**
     * 星芽号
     */
    @Field(type = FieldType.Long)
    private Long userNo;

    /**
     * 靓号
     */
    @Field(type = FieldType.Long)
    private Long niceNo;

    /**
     * 头像
     */
    private String userAvatar;

    /**
     * 注册
     */
    @Field(type = FieldType.Keyword)
    private Long userCreateTime;

    /**
     * 更新时间
     */
    @Field(type = FieldType.Long)
    private Long userUpdateTime;

    /**
     * 用户状态
     */
    @Field(type = FieldType.Integer)
    private Integer userStatus;

    /**
     * 性别
     */
    @Field(type = FieldType.Integer)
    private Integer userSex;

    /**
     * 国家id
     */
    @Field(type = FieldType.Integer)
    private Integer countryId;

    /**
     * 区域id
     */
    @Field(type = FieldType.Integer)
    private Integer areaId;

    /**
     * 财富等级
     */
    @Field(type = FieldType.Integer)
    private Integer wealthLevel;

    /**
     * 魅力等级
     */
    @Field(type = FieldType.Integer)
    private Integer charmLevel;

    /**
     * 星耀等级
     */
    @Field(type = FieldType.Integer)
    private Integer starLevel;

    /**
     * 渠道id
     */
    @Field(type = FieldType.Long)
    private Long channelId;

    /**
     * 用户手机号
     */
    @Field(type = FieldType.Keyword)
    private String userPhone;

    /**
     * 是否主播
     */
    @Field(type = FieldType.Integer)
    private Integer isAnchor;

    /**
     * 纬度
     */
    @Field(type = FieldType.Double)
    private Double userLatitude;

    /**
     * 经度
     */
    @Field(type = FieldType.Double)
    private Double userLongitude;

    /**
     * 直播状态  101：直播中  201：未开播
     */
    @Field(type = FieldType.Integer)
    private Integer liveStatus;

    /**
     * 地理位置
     */
    @GeoPointField
    private GeoPoint location;

    /**
     * 定位城市
     */
    @Field(type = FieldType.Text)
    private String locationCity;

    /**
     * 是否开通单聊权限
     */
    @Field(type = FieldType.Integer)
    private Integer singleChatEnable;

    /**
     * 是否在线
     */
    @Field(type = FieldType.Integer)
    private Integer isOnline;

    /**
     * 年龄
     */
    @Field(type = FieldType.Integer)
    private Integer userAge;

    /**
     * 单聊分类
     */
    @Field(type = FieldType.Long)
    private Long singleChatCategoryId;

    /**
     * 身份证号
     */
    @Field(type = FieldType.Keyword)
    private String idCardNo;

}
