package com.starbuds.server.service.data.impl;

import com.starbuds.server.common.constant.data.BroadcastTypeEnum;
import com.starbuds.server.common.constant.data.ConfigBusinessDictEnum;
import com.starbuds.server.common.pojo.db.BroadcastHisDO;
import com.starbuds.server.common.pojo.rtc.RtcRoom;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.BroadcastHisService;
import com.starbuds.server.common.service.data.ConfigBusinessDictService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.service.data.dao.BroadcastHisDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.junit.platform.commons.util.StringUtils;
import org.springframework.scheduling.annotation.Async;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author: lujiao
 * @Date: 2022/1/13 15:35
 */
@Service(version = "1.0.0")
@Slf4j
public class BroadcastHisServiceImpl implements BroadcastHisService {

    @Resource
    private BroadcastHisDAO broadcastHisDAO;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    ConfigBusinessDictService configService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Override
    @Async
    public void insert(Long userId, Long targetId, BroadcastTypeEnum typeEnum,
                       Long costValue, String iconUrl, Integer costNum,
                       Long roomId, Long extraId, String remark) {
        // 这里做数值判断
        Long limit = 0L;
        String limitStr = configService.getConfigDictValue(ConfigBusinessDictEnum.BROADCAST_WALL_THRESHOLD);
        if (StringUtils.isNotBlank(limitStr)) {
            limit = Long.valueOf(limitStr);
        }
        Long singlePrice = costValue / costNum;
        if (singlePrice < limit) {
            return;
        }
        BroadcastHisDO hisDO = new BroadcastHisDO();
        hisDO.setHisId(identityService.getId());
        hisDO.setUserId(userId);
        hisDO.setTargetId(targetId);
        hisDO.setShowType(typeEnum.getCode());
        hisDO.setCostValue(costValue);
        hisDO.setCostNum(costNum);
        hisDO.setIconUrl(iconUrl);
        if (roomId != null && roomId > 0) {
            RtcRoom room = rtcRoomService.getRoom(roomId);
            if (room != null) {
                hisDO.setRoomType(room.getRoomType());
                hisDO.setRoomId(roomId);
            }
        }
        hisDO.setExtraId(extraId);
        hisDO.setRemark(remark);
        hisDO.setCreateTime(System.currentTimeMillis());
        broadcastHisDAO.insert(hisDO);
    }

    @Override
    public List<BroadcastHisDO> getBroadcastHisList(Integer page, Integer pageSize) {
        Integer cursor = (page - 1) * pageSize;
        if (page * pageSize > 200) {
            return new ArrayList<>();
        }
        return broadcastHisDAO.getBroadcastHisList(cursor, pageSize);
    }
}
