package com.starbuds.server.service.data.impl;

import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.user.SwitchStatusEnum;
import com.starbuds.server.common.constant.user.SwitchTypeEnum;
import com.starbuds.server.common.pojo.data.ChatBubbleVO;
import com.starbuds.server.common.pojo.db.UserSwitchDO;
import com.starbuds.server.common.service.data.ChatBubbleService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.data.dao.ChatBubbleDAO;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

/**
 * 聊天气泡框服务实现
 *
 * @author Lay
 * @date 2021/6/17
 */
@Service(version = "1.0.0")
public class ChatBubbleServiceImpl implements ChatBubbleService {
    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    BagService bagService;

    @Resource
    ChatBubbleDAO chatBubbleDAO;

    /**
     * 查询气泡框列表
     *
     * @return
     */
    @Override
    public List<ChatBubbleVO> getBubbleList() {
        return chatBubbleDAO.selectAll();
    }

    /**
     * 根据 id 列表获取气泡框列表
     *
     * @param bubbleIds
     * @return
     */
    @Override
    public List<ChatBubbleVO> getBubbleListByIds(List<Long> bubbleIds) {
        if (CollectionUtils.isEmpty(bubbleIds)) {
            return Collections.emptyList();
        }
        return chatBubbleDAO.selectAllByIds(bubbleIds);
    }

    /**
     * 获取气泡框
     *
     * @param bubbleId
     * @return
     */
    @Override
    public ChatBubbleVO getBubble(Long bubbleId) {
        return chatBubbleDAO.selectById(bubbleId);
    }

    /**
     * 获取用户启用的气泡框
     *
     * @param userId
     * @return
     */
    @Override
    public ChatBubbleVO getUserEnabledBubble(Long userId) {
        UserSwitchDO userSwitch = userService.getSwitch(userId, SwitchTypeEnum.CHAT_BUBBLE);
        if (userSwitch != null) {
            if (SwitchStatusEnum.OPEN.getCode() == userSwitch.getSwitchStatus()) {
                Long bubbleId = userSwitch.getItemId();
                if (bubbleId != null) {
                    // 判断坐骑是否过期
                    if (bagService.getItemQuantity(userId, BagItemTypeEnum.CHAT_BUBBLE, bubbleId) > 0) {
                        return getBubble(bubbleId);
                    }
                }
            }
        }
        return null;
    }

    /**
     * 判断气泡框是否存在
     *
     * @param bubbleId
     * @return
     */
    @Override
    public boolean isExists(Long bubbleId) {
        return chatBubbleDAO.countById(bubbleId) > 0;
    }
}
