package com.starbuds.server.service.data.impl;

import com.starbuds.server.common.constant.data.DataGreenTypeEnum;
import com.starbuds.server.common.constant.data.GreenBizTypeEnum;
import com.starbuds.server.common.pojo.data.green.GreenAuditTrace;
import com.starbuds.server.common.service.data.DataGreenService;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import com.starbuds.server.service.data.provider.AbstractGreenProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.context.ApplicationContext;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

/**
 * 内容审核
 *
 * @author aoe
 * @date 2021/8/23
 */
@Service(version = "1.0.0")
@Slf4j
public class DataGreenServiceImpl implements DataGreenService {

    private Map<String, AbstractGreenProvider> greenProviderBeanMap = new HashMap<>();

    @Reference(version = "1.0.0")
    private ReleaseConfigService releaseConfigService;

    @Resource
    ApplicationContext context;

    private AbstractGreenProvider greenProvider() {
        return greenProviderBeanMap.get(releaseConfigService.getReleaseConfig("green_provider"));
    }

    @PostConstruct
    private void init() {
        Map<String, AbstractGreenProvider> map = context.getBeansOfType(AbstractGreenProvider.class);
        map.forEach((key, value) -> greenProviderBeanMap.put(value.companyType().getValue(), value));
    }

    @Override
    public GreenAuditTrace imgAudit(String imageUrl, String uniqueID, Long bizId, GreenBizTypeEnum bizType) {
        return this.greenProvider().imgAudit(imageUrl, uniqueID, bizId, bizType, DataGreenTypeEnum.IMAGE);
    }

    @Override
    public GreenAuditTrace textAudit(String text, String uniqueID, Long bizId, GreenBizTypeEnum bizType) {
        return this.greenProvider().textAudit(text, uniqueID, bizId, bizType, DataGreenTypeEnum.TEXT);
    }
}
