package com.starbuds.server.service.data.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.common.constant.common.AppType;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.ClientType;
import com.starbuds.server.common.constant.data.BaiduATypeEnum;
import com.starbuds.server.common.constant.data.HandleStatusEnum;
import com.starbuds.server.common.constant.finance.ScoreTypeEnum;
import com.starbuds.server.common.constant.user.FeedbackTypeEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.data.*;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.user.FeedbackTypeVO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.DataService;
import com.starbuds.server.common.service.user.ChannelService;
import com.starbuds.server.service.data.dao.*;
import com.starbuds.server.service.data.provider.BaiduProvider;
import com.starbuds.server.service.data.provider.ImageGreenProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 数据服务实现
 *
 * @author Lay
 * @date 2019/12/25
 */
@Service(version = "1.0.0")
@Slf4j
public class DataServiceImpl implements DataService {

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    ChannelService channelService;

    @Resource
    DataLevelScoreDAO dataLevelScoreDAO;

    @Resource
    DataReportReasonDAO dataReportReasonDAO;

    @Resource
    ReportDAO reportDAO;

    @Resource
    DataForbiddenWordOptionDAO dataForbiddenWordOptionDAO;

    @Resource
    DataFeedbackDAO dataFeedbackDAO;

    @Resource
    AppVersionDAO appVersionDAO;

    @Resource
    DataLiveTagDAO dataLiveTagDAO;

    @Resource
    BannerDAO bannerDAO;

    @Resource
    DataSingleChatEvaluateTagDAO dataSingleChatEvaluateTagDAO;

    @Resource
    DataUserTagDAO dataUserTagDAO;

    @Resource
    DataHobbyTagDAO dataHobbyTagDAO;

    @Resource
    DataBaiduNoticeDAO dataBaiduNoticeDAO;

    @Resource
    DataCountryDAO dataCountryDAO;

    @Resource
    PhoneAreaCodeDAO phoneAreaCodeDAO;

    @Resource
    DataCountryEnDAO dataCountryEnDAO;

    @Resource
    PosterImageDAO posterImageDAO;

    @Resource
    DataLiveRateDAO dataLiveRateDAO;

    @Resource
    DataRtcRoomBackgroundDAO dataRtcRoomBackgroundDAO;

    @Resource
    RtcRoomCategoryDAO rtcRoomCategoryDAO;

    @Resource
    ImageGreenProvider imageGreenProvider;

    @Resource
    DataBankDAO dataBankDAO;

    @Resource
    DataGreetTemplateDAO dataGreetTemplateDAO;

    @Resource
    PlayTopDAO playTopDAO;

    @Resource
    GradeConfigDAO gradeConfigDAO;

    @Resource
    private RoomMsgTypesDAO roomMsgTypesDAO;

    /**
     * 获取积分等级
     *
     * @param score
     * @param scoreType
     * @return
     */
    @Override
    public Integer getScoreLevel(Long score, ScoreTypeEnum scoreType) {
        return Optional.ofNullable(dataLevelScoreDAO.selectLevelByValue(scoreType.getValue(), score)).orElse(1);
    }

    /**
     * 获取积分等级默认0级
     *
     * @param score
     * @param scoreType
     * @return
     */
    @Override
    public Integer getScoreLevelDefault(Long score, ScoreTypeEnum scoreType) {
        return Optional.ofNullable(dataLevelScoreDAO.selectLevelByValue(scoreType.getValue(), score)).orElse(0);
    }

    /**
     * 查询下一等级
     *
     * @param level
     * @param scoreType
     * @return
     */
    @Override
    public DataLevelScoreDO getNextLevelScore(int level, ScoreTypeEnum scoreType) {
        return dataLevelScoreDAO.selectNextLevel(level, scoreType.getValue());
    }

    /**
     * 查询当前等级
     *
     * @param level
     * @param scoreType
     * @return
     */
    @Override
    public DataLevelScoreDO getCurrentLevelScore(int level, ScoreTypeEnum scoreType) {
        return dataLevelScoreDAO.selectCurrentLevel(level, scoreType.getValue());
    }

    /**
     * 查询最后的等级
     *
     * @param scoreType
     * @return
     */
    @Override
    public DataLevelScoreDO getLastLevelScore(ScoreTypeEnum scoreType) {
        return dataLevelScoreDAO.selectLastLevelScore(scoreType.getValue());
    }

    /**
     * 查询活动勋章列表
     *
     * @return
     */
    @Override
    public List<DataLevelScoreDO> getActivitiesMedalList() {
        return dataLevelScoreDAO.selectActivitiesMedalList();
    }

    /**
     * 获取等级积分
     *
     * @param level
     * @param scoreType
     * @return
     */
    @Override
    public Long getLevelScore(Integer level, ScoreTypeEnum scoreType) {
        return dataLevelScoreDAO.selectScoreByLevel(scoreType.getValue(), level);
    }

    /**
     * 获取举报原因列表
     *
     * @return
     */
    @Override
    public List<DataReportReasonVO> getReportReasonList() {
        List<DataReportReasonDO> list = dataReportReasonDAO.selectAll();

        if (CollectionUtil.isNotEmpty(list)) {
            List<DataReportReasonVO> retList = new ArrayList<>();
            for (DataReportReasonDO reasonDO : list) {
                DataReportReasonVO reasonVO = new DataReportReasonVO();
                BeanUtils.copyProperties(reasonDO, reasonVO);
                retList.add(reasonVO);
            }
            return retList;
        }

        return Collections.emptyList();
    }

    /**
     * 创建举报
     *
     * @param reportAddDTO
     * @return
     */
    @Override
    public Long addReport(ReportAddDTO reportAddDTO) {
        Long reportId = identityService.getId();

        try {
            ReportDO reportDO = new ReportDO();
            BeanUtils.copyProperties(reportAddDTO, reportDO);
            reportDO.setReportId(reportId);
            reportDO.setHandleStatus(HandleStatusEnum.WAIT.getValue());
            reportDO.setReportCreateTime(System.currentTimeMillis());

            reportDAO.insert(reportDO);

            return reportId;
        } catch (Exception e) {
            log.error("举报失败", e);
        }

        return null;
    }

    /**
     * 判断举报原因是否存在
     *
     * @param reasonId
     * @return
     */
    @Override
    public boolean isReportReasonExists(Long reasonId) {
        return dataReportReasonDAO.countById(reasonId) > 0;
    }

    /**
     * 处理报告
     *
     * @param reportId      报告id
     * @param adminId       管理员id
     * @param handleRemarks 处理意见
     * @param statusEnum    状态
     * @return 是否成功
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean handleReport(Long reportId, Long adminId, String handleRemarks, HandleStatusEnum statusEnum) {
        ReportDO report = new ReportDO();
        report.setReportId(reportId);
        report.setAdminId(adminId);
        report.setHandleRemarks(handleRemarks);
        report.setHandleStatus(statusEnum.getValue());
        report.setHandleTime(System.currentTimeMillis());
        return reportDAO.updateById(report) > 0;
    }

    /**
     * 判断举报记录是否存在
     *
     * @param reportId 报告id
     * @return 是否存在
     */
    @Override
    public boolean isReportExists(Long reportId) {
        return reportDAO.countByReportId(reportId) > 0;
    }

    /**
     * 获取report用户id
     *
     * @param reportId 报告id
     * @return 用户id
     */
    @Override
    public Long getReportUserId(Long reportId) {
        return reportDAO.getReportUserId(reportId);
    }

    /**
     * 获取禁言选项
     *
     * @return
     */
    @Override
    public List<DataForbiddenWordOptionVO> getForbiddenWordOptions() {
        return dataForbiddenWordOptionDAO.selectAll();
    }

    /**
     * 根据 id 查询禁言选项
     *
     * @param optionId
     * @return
     */
    @Override
    public DataForbiddenWordOptionVO getForbiddenWordOptionById(Long optionId) {
        return dataForbiddenWordOptionDAO.selectById(optionId);
    }

    /**
     * 提交意见
     *
     * @param feedbackDO
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean submitSuggest(DataFeedbackDO feedbackDO) {
        return dataFeedbackDAO.insert(feedbackDO) == 1;
    }

    /**
     * 获取意见反馈详情
     *
     * @param feedId
     * @return
     */
    @Override
    public DataFeedbackVO getSuggestDetail(Long feedId) {
        return dataFeedbackDAO.selectVOById(feedId);
    }

    /**
     * 获取我的意见反馈列表
     *
     * @param userId
     * @param feedType
     * @param handleStatus
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public PageList<DataFeedbackVO> getFeedbackPageList(Long userId, Integer feedType, Integer handleStatus, int pageNum, int pageSize) {
        PageList<DataFeedbackVO> page = new PageList<>(pageNum, pageSize);
        page.setList(dataFeedbackDAO.getFeedbackPageList(userId, feedType, handleStatus, page.getPageCursor(), page.getPageSize()));
        page.setTotal(dataFeedbackDAO.countFeedbackPageList(userId, feedType, handleStatus));
        return page;
    }

    /**
     * 获取意见反馈类型
     *
     * @return
     */
    @Override
    public List<FeedbackTypeVO> getFeedbackTypeList() {
        List<FeedbackTypeVO> voList = new ArrayList<>();
        for (FeedbackTypeEnum item : FeedbackTypeEnum.values()) {
            voList.add(new FeedbackTypeVO(item.getCode(), item.getDesc()));
        }
        return voList;
    }

    /**
     * 获取应用最新版本
     *
     * @param clientType
     * @param appType
     * @param versionNum
     * @param channelId
     * @return
     */
    @Override
    public AppVersionDO getLatestAppVersion(ClientType clientType, AppType appType, long versionNum, Long channelId) {
        Long cid = channelService.getAppVersionChannelId(channelId);
        AppVersionDO appVersion = appVersionDAO.selectLatest(clientType.getValue(), appType.getValue(), versionNum, cid);
        if (appVersion != null) {
            // 判断是否强更
            if (BoolIntVal.False == Optional.ofNullable(appVersion.getVersionIsForce()).orElse(BoolIntVal.False)) {
                // 非强更的情况下，需要判断两个版本间是否隔着强更版本
                // 如果两个版本间隔着强更版本，那当前版本即为强更
                Long latestForceVersionNum = appVersionDAO.selectLatestForceVersionNum(clientType.getValue(), appType.getValue(), versionNum, cid);
                if (latestForceVersionNum != null && latestForceVersionNum < appVersion.getVersionNum()) {
                    appVersion.setVersionIsForce(BoolIntVal.True);
                }
            }
        }

        return appVersion;
    }

    /**
     * 根据 id 查询直播标签
     *
     * @param tagIds
     * @return
     */
    @Override
    public List<DataLiveTagDO> getLiveTagByIds(List<Long> tagIds) {
        return dataLiveTagDAO.selectByIds(tagIds);
    }

    /**
     * 获取 banner 列表
     *
     * @param bannerPosition
     * @param categoryId
     * @param channelId
     * @return
     */
    @Override
    public List<BannerVO> getBanners(Integer bannerPosition, Long categoryId, Long channelId) {
        Long cid = channelService.getBannerChannelId(channelId);
        return bannerDAO.selectAll(bannerPosition, System.currentTimeMillis(), categoryId, cid);
    }

    /**
     * 获取单聊评价标签
     *
     * @param tagType
     * @return
     */
    @Override
    public List<SingleChatEvaluateTagVO> getSingleChatEvaluateTags(Integer tagType) {
        return dataSingleChatEvaluateTagDAO.selectAll(tagType);
    }

    /**
     * 获取用户标签
     *
     * @param tagType
     * @return
     */
    @Override
    public List<UserTagVO> getUserTags(Integer tagType) {
        return dataUserTagDAO.selectAll(tagType);
    }

    /**
     * 获取兴趣标签
     *
     * @param tagType
     * @return
     */
    @Override
    public List<HobbyTagVO> getHobbyTags(Integer tagType) {
        return dataHobbyTagDAO.selectAll(tagType);
    }

    /**
     * 判断标签是否存在
     *
     * @param tagIds
     * @return
     */
    @Override
    public boolean isSingleChatTagIdsExists(List<Long> tagIds) {
        return dataSingleChatEvaluateTagDAO.countByIds(tagIds) == tagIds.size();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean saveBaiduNotice(BaiduNoticeDO androidDO) {
        return dataBaiduNoticeDAO.insert(androidDO) == 1;
    }

    @Override
    public boolean baiduCallback(String callbackUrl, String akey, BaiduATypeEnum aTypeEnum, String aValue) {
        return BaiduProvider.callback(callbackUrl, akey, aTypeEnum.getAType(), aValue);
    }

    @Override
    public BaiduNoticeDTO getBaiduNotice(String idfa, String imeiMd5) {
        return dataBaiduNoticeDAO.selectBaiduNotice(idfa, imeiMd5);
    }

    @Override
    public List<CountryPhoneAreaCodeVO> getCountryPhoneCode() {
        return dataCountryDAO.selectCountryPhoneCode();
    }

    @Override
    public List<PhoneAreaCodeDO> getPhoneAreaCode() {
        return dataCountryDAO.selectPhoneAreaCode();
    }

    @Override
    public PhoneAreaCodeDO getPhoneAreaCodeById(Integer countryId) {
        return dataCountryDAO.selectPhoneAreaCodeById(countryId);
    }

    @Override
    public Integer getMarketStatus(String versionCode, Long channelId) {
        return appVersionDAO.selectMarketStatus(versionCode, channelId);
    }

    @Override
    public String getAreaCode(Integer countryId) {
        String areaCode = phoneAreaCodeDAO.selectAreaCodeById(countryId);
        return StringUtils.isEmpty(areaCode) ? "86" : areaCode;
    }

    @Override
    public DataCountryDO getCountry(Integer countryId) {
        return dataCountryDAO.selectById(countryId);
    }

    /**
     * 根据国家中文名获取英文名
     *
     * @param chineseName
     * @return
     */
    @Override
    public String getCountryNameEn(String chineseName) {
        return dataCountryEnDAO.selectEnglishNameByChineseName(chineseName);
    }

    @Override
    public List<PosterImageDO> getPosterImages(Long channelId, Integer imageType) {

        // 判断跟随渠道id
        Long posterImageChannelId = channelService.getPosterImageChannelId(channelId);
        if (posterImageChannelId != null) {
            channelId = posterImageChannelId;
        }

        return posterImageDAO.selectPosterImages(channelId, imageType);
    }

    @Override
    public List<BannerVO> getOpenScreenBanners(Integer bannerPosition, Long categoryId, Long channelId) {
        Long cid = channelService.getBannerChannelId(channelId);
        return bannerDAO.selectAllLimitOne(bannerPosition, System.currentTimeMillis(), categoryId, cid);
    }

    @Override
    public boolean updateBaiduNotice(Long noticeId, String atype, String avalue) {
        return dataBaiduNoticeDAO.updateById(noticeId, atype, avalue) > 0;
    }

    @Override
    public ImageGreenAuditResult scanImage(String url, long dataId) {
        return imageGreenProvider.scanImage(url, dataId);
    }

    /**
     * 获取直播码率配置
     *
     * @return
     */
    @Override
    public List<DataLiveRateDO> getLiveRates() {
        return dataLiveRateDAO.selectAll();
    }

    /**
     * 获取语聊房壁纸列表
     *
     * @return
     */
    @Override
    public List<RtcRoomBackground> getRtcRoomBackgrounds(Long roomId) {
        List<RtcRoomBackground> targetBackgrounds = dataRtcRoomBackgroundDAO.selectAll().stream().filter(m -> {
            if (m.getScope() == 0) {
                return true;
            }

            if (roomId == 0) {
                return false;
            }

            if (System.currentTimeMillis() > m.getEndTimestamp()) {
                return false;
            }

            if (StringUtils.isNotBlank(m.getScopeValue())) {
                String strRoomId = String.valueOf(roomId);
                return Arrays.stream(m.getScopeValue().split(",")).anyMatch(item -> item.contentEquals(strRoomId));
            }

            return false;
        }).collect(Collectors.toList());

        return targetBackgrounds;
    }

    /**
     * 房间背景是否存在
     *
     * @param backgroundId
     * @return
     */
    @Override
    public boolean isRtcRoomBackgroundExists(Integer backgroundId) {
        return dataRtcRoomBackgroundDAO.countById(backgroundId) > 0;
    }

    /**
     * 判断房间类目是否存在
     *
     * @param categoryId
     * @return
     */
    @Override
    public boolean isRtcRoomCategoryExists(Long categoryId) {
        return rtcRoomCategoryDAO.countById(categoryId) > 0;
    }

    /**
     * 获取房间背景
     *
     * @param backgroundId
     * @return
     */
    @Override
    public RtcRoomBackground getRtcRoomBackground(Integer backgroundId) {
        return dataRtcRoomBackgroundDAO.selectById(backgroundId);
    }

    /**
     * 通过银行编码获取银行
     *
     * @param bankCode
     * @return
     */
    @Override
    public DataBankDO getBankByBankCode(String bankCode) {
        return dataBankDAO.selectBankByBankCode(bankCode);
    }

    /**
     * 通过银行编码获取银行
     *
     * @param bankCodes
     * @return
     */
    @Override
    public Map<String, DataBankDO> getBanksByBankCodes(Set<String> bankCodes) {
        if (CollectionUtil.isEmpty(bankCodes)) {
            return new HashMap<>();
        }
        return dataBankDAO.selectBanksByBankCodes(bankCodes);
    }

    /**
     * 获取招呼模板
     *
     * @param greetType
     * @return
     */
    @Override
    public String getGreetContent(Integer greetType) {
        return dataGreetTemplateDAO.selectRandomTemplate(greetType);
    }

    /**
     * 添加陪玩头条（陪玩 改 开黑）
     *
     * @param topDO
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int addPlayTop(PlayTopDO topDO) {
        return playTopDAO.insert(topDO);
    }

    /**
     * 获取陪玩头条（陪玩 改 开黑）
     *
     * @param pageSize
     * @return
     */
    @Override
    public List<PlayTopItem> getPlayTop(int pageSize) {
        return playTopDAO.selectList(pageSize);
    }

    /**
     * 获取评分配置列表
     *
     * @param gradeType 评分类型
     * @return
     */
    @Override
    public List<GradeConfigDO> getGradeConfigs(Integer gradeType, String des) {
        return gradeConfigDAO.getGradeConfigList(gradeType, des);
    }

    /**
     * 更新评级表信息
     *
     * @param id     评级记录ID
     * @param weight 权重
     * @param enable 是否启用 1：启用 0：禁用
     * @return
     */
    @Override
    public int updateGradeConfig(long id, int weight, int enable) {
        return gradeConfigDAO.updateGradeConfig(id, weight, enable, System.currentTimeMillis());
    }

    /**
     * 获取房间消息列表
     *
     * @return
     */
    @Override
    public List<String> getRoomTypeList() {
        return roomMsgTypesDAO.getRoomTypeList();
    }
}
