package com.starbuds.server.service.data.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.user.SwitchStatusEnum;
import com.starbuds.server.common.constant.user.SwitchTypeEnum;
import com.starbuds.server.common.pojo.data.HeadwearVO;
import com.starbuds.server.common.pojo.data.MedalVO;
import com.starbuds.server.common.pojo.db.UserSwitchDO;
import com.starbuds.server.common.service.data.MedalService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.data.dao.MedalDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;
import java.util.Optional;

/**
 * 勋章服务实现
 *
 * @author Lay
 * @date 2020/1/12
 */
@Service(version = "1.0.0")
@Slf4j
public class MedalServiceImpl implements MedalService {

    @Autowired
    MedalDAO medalDAO;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Reference(version = "1.0.0")
    private BagService bagService;

    @Override
    public List<MedalVO> getMedalList() {
        return medalDAO.selectAll();
    }

    /**
     * 根据 id 列表查询勋章列表
     *
     * @param medalIds
     * @return
     */
    @Override
    public List<MedalVO> getMedalListByIds(List<Long> medalIds) {
        return medalDAO.selectAllByIds(medalIds);
    }

    /**
     * 获得已启用勋章
     *
     * @param userId
     * @return
     */
    @Override
    public MedalVO getEnabledMedal(Long userId) {
        Long medalId = getEnabledMedalId(userId);
        if (medalId == null) {
            return null;
        }
        return medalDAO.selectById(medalId);
    }

    @Override
    public boolean isExists(Long medalId) {
        return medalDAO.countById(medalId) > 0;
    }

    /**
     * 获取用户启用的勋章id
     *
     * @param userId
     * @return
     */
    @Override
    public Long getEnabledMedalId(Long userId) {
        UserSwitchDO userSwitch = userService.getSwitch(userId, SwitchTypeEnum.MEDAL);
        if (userSwitch == null) {
            return null;
        }
        if (SwitchStatusEnum.OPEN.getCode() != userSwitch.getSwitchStatus()) {
            return null;
        }
        Long medalId = userSwitch.getItemId();
        if (medalId == null) {
            return null;
        }
        // 判断勋章是否过期
        if (bagService.getItemQuantity(userId, BagItemTypeEnum.MEDAL, medalId) > 0) {
            return medalId;
        }
        return null;
    }

    /**
     * 获取勋章
     *
     * @param medalId
     * @return
     */
    @Override
    public MedalVO getMedal(Long medalId) {
        return medalDAO.selectById(medalId);
    }
}
