package com.starbuds.server.service.data.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.common.pojo.data.UserModuleVO;
import com.starbuds.server.common.pojo.db.UserModuleDO;
import com.starbuds.server.common.service.data.ModuleService;
import com.starbuds.server.service.data.dao.UserModuleDAO;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @Author jiangxiaowei
 * @DateTime 2021/3/1 14:24
 * @Description 模块服务
 */
@Service(version = "1.0.0")
public class ModuleServiceImpl implements ModuleService {

    @Resource
    UserModuleDAO userModuleDAO;

    /**
     * 获取用户模块
     *
     * @return
     */
    @Override
    public List<UserModuleVO> getUserModules() {
        List<UserModuleVO> modules = userModuleDAO.selectList();
        if (CollectionUtil.isNotEmpty(modules)) {
            Map<Long, UserModuleVO> moduleMap = modules.stream().filter(e -> 0L == e.getParentId()).collect(Collectors.toMap(UserModuleVO::getModuleId, Function.identity()));
            Map<Long, List<UserModuleVO>> masterChildModules = modules.stream().collect(Collectors.groupingBy(UserModuleVO::getParentId));
            for (Long masterId : moduleMap.keySet()) {
                moduleMap.get(masterId).setChildModules(masterChildModules.get(masterId));
            }
            return moduleMap.values().stream().collect(Collectors.toList());
        }
        return Collections.EMPTY_LIST;
    }

    /**
     * 根据类型获取模块id
     *
     * @param contentType
     * @return
     */
    @Override
    public Set<Long> getModuleIdsByType(Integer contentType) {
        return userModuleDAO.selectModuleIdsByType(contentType);
    }

    /**
     * 获取模块
     *
     * @param moduleIds
     * @return
     */
    @Override
    public Map<Long, UserModuleDO> getModules(Set<Long> moduleIds) {
        if (CollectionUtil.isEmpty(moduleIds)) {
            return Collections.EMPTY_MAP;
        }
        return userModuleDAO.selectMap(moduleIds);
    }

}
