package com.starbuds.server.service.data.impl;

import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.user.SwitchStatusEnum;
import com.starbuds.server.common.constant.user.SwitchTypeEnum;
import com.starbuds.server.common.pojo.data.MountsVO;
import com.starbuds.server.common.pojo.db.MountsDO;
import com.starbuds.server.common.pojo.db.UserSwitchDO;
import com.starbuds.server.common.service.data.MountsService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.data.dao.MountsDAO;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 坐骑服务实现
 *
 * @author Lay
 * @date 2020/1/12
 */
@Service(version = "1.0.0")
public class MountsServiceImpl implements MountsService {

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    BagService bagService;

    @Resource
    MountsDAO mountsDAO;

    /**
     * 根据 id 列表获取坐骑列表
     *
     * @param mountsIds
     * @return
     */
    @Override
    public List<MountsVO> getMountsListByIds(List<Long> mountsIds) {
        return mountsDAO.selectAllByIds(mountsIds);
    }

    /**
     * 获取坐骑
     *
     * @param mountsId
     * @return
     */
    @Override
    public MountsVO getMounts(Long mountsId) {
        if (mountsId != null) {
            MountsDO mountsDO = mountsDAO.selectById(mountsId);
            if (mountsDO != null) {
                MountsVO mountsVO = new MountsVO();
                BeanUtils.copyProperties(mountsDO, mountsVO);
                return mountsVO;
            }
        }
        return null;
    }

    /**
     * 获取用户启用的坐骑
     *
     * @param userId
     * @return
     */
    @Override
    public MountsVO getUserEnabledMounts(Long userId) {
        UserSwitchDO userSwitch = userService.getSwitch(userId, SwitchTypeEnum.MOUNTS);
        if (userSwitch != null) {
            if (SwitchStatusEnum.OPEN.getCode() == userSwitch.getSwitchStatus()) {
                Long mountsId = userSwitch.getItemId();
                if (mountsId != null) {
                    // 判断坐骑是否过期
                    if (bagService.getItemQuantity(userId, BagItemTypeEnum.MOUNTS, mountsId) > 0) {
                        return getMounts(mountsId);
                    }
                }
            }
        }

        return null;
    }

    /**
     * 判断坐骑是否存在
     *
     * @param mountsId
     * @return
     */
    @Override
    public boolean isExists(Long mountsId) {
        return mountsDAO.countById(mountsId) > 0;
    }
}
