package com.starbuds.server.service.data.impl;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.data.MusicStatusEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.data.MusicVO;
import com.starbuds.server.common.pojo.db.MusicDO;
import com.starbuds.server.common.service.data.MusicSearchService;
import com.starbuds.server.service.data.dao.MusicDAO;
import com.starbuds.server.service.data.es.dao.MusicRepository;
import com.starbuds.server.service.data.es.entity.MusicEntity;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Service;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/8/3 19:04
 * @Description 音乐搜索
 */
@Slf4j
@Service(version = "1.0.0")
public class MusicSearchServiceImpl implements MusicSearchService {

    @Resource
    MusicRepository musicRepository;

    @Resource
    MusicDAO musicDAO;

    /**
     * 保存或修改
     *
     * @param musicDO
     * @return
     */
    @Override
    public boolean saveOrUpdate(MusicDO musicDO) {

        if (musicDO != null) {
            // 待写入es的对象
            MusicEntity entity = new MusicEntity();
            // 从db查询
            MusicDO dbMusic = musicDAO.selectById(musicDO.getMusicId());
            // 如果db存在,给entity赋值
            if (dbMusic != null) {
                BeanUtils.copyProperties(dbMusic, entity);
            }
            // 其他字段以传入为准
            if (StringUtils.isNotEmpty(musicDO.getMusicName())) {
                entity.setMusicName(musicDO.getMusicName());
            }
            if (StringUtils.isNotEmpty(musicDO.getMusicSinger())) {
                entity.setMusicSinger(musicDO.getMusicSinger());
            }
            if (StringUtils.isNotEmpty(musicDO.getMusicUrl())) {
                entity.setMusicUrl(musicDO.getMusicUrl());
            }
            if (musicDO.getOriginal() != null) {
                entity.setOriginal(musicDO.getOriginal());
            }
            if (musicDO.getDownloadCount() != null) {
                entity.setDownloadCount(musicDO.getDownloadCount());
            }
            if (musicDO.getUpdateTime() != null) {
                entity.setUpdateTime(musicDO.getUpdateTime());
            }
            if (musicDO.getStatus() != null) {
                entity.setStatus(musicDO.getStatus());
            }
            if (musicDO.getShelfStatus() != null) {
                entity.setShelfStatus(musicDO.getShelfStatus());
            }
            if (StringUtils.isNotEmpty(musicDO.getShelfRemark())) {
                entity.setShelfRemark(musicDO.getShelfRemark());
            }

            musicRepository.save(entity);
            return true;
        }
        log.info("对象不能为空");
        return false;
    }

    /**
     * 搜索音乐
     *
     * @param content
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<MusicVO> searchMusic(String content, int pageIndex, int pageSize) {
        PageList<MusicVO> page = new PageList<>(pageIndex, pageSize);
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        if (StringUtils.isNotEmpty(content)) {
            BoolQueryBuilder shouldQueryBuilder = QueryBuilders.boolQuery();
            shouldQueryBuilder.should(QueryBuilders.matchQuery("musicName", content));
            shouldQueryBuilder.should(QueryBuilders.wildcardQuery("musicName", String.format("*%s*", content)));
            shouldQueryBuilder.should(QueryBuilders.matchQuery("musicSinger", content));
            shouldQueryBuilder.should(QueryBuilders.wildcardQuery("musicSinger", String.format("*%s*", content)));
            boolQueryBuilder.must(shouldQueryBuilder);
        }
        boolQueryBuilder.must(QueryBuilders.matchQuery("shelfStatus", 1));
        boolQueryBuilder.must(QueryBuilders.rangeQuery("status").gt(MusicStatusEnum.USER_DEL.getValue()));
        log.info("查询语句：{}", boolQueryBuilder.toString());
        PageRequest pageRequest = PageRequest.of(pageIndex - 1, pageSize, Sort.by(Sort.Direction.DESC, "updateTime"));
        Page<MusicEntity> searchResult = musicRepository.search(boolQueryBuilder, pageRequest);
        List<MusicEntity> resultContent = searchResult.getContent();
        log.info("es返回的结果内容：{}", JSON.toJSONString(resultContent, true));

        page.setTotal(Convert.toInt(searchResult.getTotalElements()));

        List<MusicVO> respList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(resultContent)) {
            List<Long> musicIds = resultContent.stream().map(MusicEntity::getMusicId).collect(Collectors.toList());
            respList = musicDAO.selectByIds(musicIds);
        }
        page.setList(respList);
        return page;
    }
}
