package com.starbuds.server.service.data.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.common.constant.data.PropsTypeEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.data.PropsVO;
import com.starbuds.server.common.pojo.db.GiftDO;
import com.starbuds.server.common.pojo.db.PropsDO;
import com.starbuds.server.common.pojo.media.bo.PriceInfoBO;
import com.starbuds.server.common.service.data.PropsService;
import com.starbuds.server.service.data.dao.PropsDAO;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 道具服务实现
 *
 * @author Lay
 * @date 2020/1/12
 */
@Service(version = "1.0.0")
public class PropsServiceImpl implements PropsService {

    @Autowired
    PropsDAO propsDAO;

    /**
     * 根据 id 列表获取道具列表
     *
     * @param propsIds
     * @return
     */
    @Override
    public List<PropsVO> getPropsByIds(List<Long> propsIds) {
        if (CollectionUtils.isEmpty(propsIds)) {
            return Collections.emptyList();
        }
        List<PropsDO> propsDOList = propsDAO.selectAllByIds(propsIds);
        if (CollectionUtil.isNotEmpty(propsDOList)) {
            List<PropsVO> list = new ArrayList<>();
            for (PropsDO propsDO : propsDOList) {
                PropsVO propsVO = new PropsVO();
                BeanUtils.copyProperties(propsDO, propsVO);
                list.add(propsVO);
            }
            return list;
        }
        return Collections.emptyList();
    }

    /**
     * 根据道具类型获取道具 id
     *
     * @param propsType
     * @return
     */
    @Override
    public List<Long> getPropsIdsByType(PropsTypeEnum propsType) {
        return propsDAO.selectIdsByType(propsType.getValue());
    }

    @Override
    public PropsVO getProps(Long propsId) {
        return propsDAO.selectVO(propsId);
    }

    @Override
    public Map<Long, PriceInfoBO> getPropsPriceMap(Collection<Long> propsIds) {
        if (CollectionUtils.isEmpty(propsIds)) {
            return Collections.emptyMap();
        }
        List<Long> ids = propsIds.stream().filter(Objects::nonNull).distinct().collect(Collectors.toList());
        Map<Long, PropsDO> map = propsDAO.selectPriceMapByIds(ids);
        Map<Long, PriceInfoBO> priceMap = new HashMap<>();
        ids.stream()
                .forEach(propsId -> {
                    PropsDO props = map.get(propsId);
                    if (props == null) {
                        return;
                    }
                    PriceInfoBO bo = new PriceInfoBO();
                    bo.setPrice(props.getPropsExPrice());
                    bo.setType(WalletTypeEnum.toEnum(props.getPropsExPriceType()));
                    priceMap.put(propsId, bo);
                });
        return priceMap;
    }

    /**
     * 道具Map
     *
     * @param propsIds
     * @return propsId:PropsVO
     */
    @Override
    public Map<Long, PropsVO> getPropsVOMap(List<Long> propsIds) {
        List<PropsVO> list = getPropsByIds(propsIds);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream()
                .filter(Objects::nonNull)
                .filter(vo -> vo.getPropsId() != null)
                .collect(Collectors.toMap(PropsVO::getPropsId, Function.identity()));
    }
}
