package com.starbuds.server.service.data.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.data.RtcRoomSearchItem;
import com.starbuds.server.common.pojo.data.RtcRoomSearchReq;
import com.starbuds.server.common.pojo.db.RtcRoomDO;
import com.starbuds.server.common.pojo.rtc.RtcRoomCategory;
import com.starbuds.server.common.pojo.rtc.RtcRoomListItem;
import com.starbuds.server.common.pojo.user.UserDTO;
import com.starbuds.server.common.pojo.user.UserPageListQO;
import com.starbuds.server.common.service.data.RtcRoomSearchService;
import com.starbuds.server.common.service.rtc.RtcBattleService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.data.dao.RtcRoomCategoryDAO;
import com.starbuds.server.service.data.dao.RtcRoomDAO;
import com.starbuds.server.service.data.es.dao.RtcRoomRepository;
import com.starbuds.server.service.data.es.entity.RtcRoomEntity;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Method;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import static com.starbuds.server.common.pojo.util.PageListUtil.nextPage;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/07/11 11:12
 * @Description 语聊房间服务
 */
@Slf4j
@Service(version = "1.0.0", methods = {@Method(name = "saveOrUpdate", timeout = 5000)})
public class RtcRoomSearchServiceImpl implements RtcRoomSearchService {

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    RtcBattleService rtcBattleService;

    @Resource
    RtcRoomRepository rtcRoomRepository;

    @Resource
    RtcRoomDAO rtcRoomDAO;

    @Reference(version = "1.0.0")
    UserService userService;

    @Resource
    RtcRoomCategoryDAO rtcRoomCategoryDAO;

    @Override
    public boolean saveOrUpdate(RtcRoomDO roomDO) {

        if (roomDO != null) {
            // 待写入es的对象
            RtcRoomEntity entity = new RtcRoomEntity();
            // 数据库获取
            RtcRoomDO dbRtcRoom = rtcRoomDAO.selectById(roomDO.getRoomId());
            // 如果db存在
            if (dbRtcRoom != null) {
                // 给entity赋值
                BeanUtils.copyProperties(dbRtcRoom, entity);
            }
            entity.setRoomId(roomDO.getRoomId());

            // 更新的字段以传入的为准
            if (StringUtils.isNotEmpty(roomDO.getRoomName())) {
                entity.setRoomName(roomDO.getRoomName());
            }
            if (roomDO.getUserId() != null) {
                entity.setUserId(roomDO.getUserId());
                entity.setUserNo(userService.getUserNo(entity.getUserId()));
            }
            if (roomDO.getRoomType() != null) {
                entity.setRoomType(roomDO.getRoomType());
            }
            if (roomDO.getRoomNo() != null) {
                entity.setRoomNo(roomDO.getRoomNo());
            }
            if (roomDO.getCategoryId() != null) {
                entity.setCategoryId(roomDO.getCategoryId());
            }
            if (StringUtils.isNotEmpty(roomDO.getRoomTitle())) {
                entity.setRoomTitle(roomDO.getRoomTitle());
            }
            if (StringUtils.isNotEmpty(roomDO.getRoomCover())) {
                entity.setRoomCover(roomDO.getRoomCover());
            }
            if (roomDO.getCategoryId() != null) {
                entity.setCategoryId(roomDO.getCategoryId());
            }
            if (StringUtils.isNotEmpty(roomDO.getRoomRule())) {
                entity.setRoomRule(roomDO.getRoomRule());
            }
            if (StringUtils.isNotEmpty(roomDO.getRoomWelcome())) {
                entity.setRoomWelcome(roomDO.getRoomWelcome());
            }
            if (StringUtils.isNotEmpty(roomDO.getRoomLockPass())) {
                entity.setRoomLockPass(roomDO.getRoomLockPass());
            }
            if (roomDO.getRoomMicFree() != null) {
                entity.setRoomMicFree(roomDO.getRoomMicFree());
            }
            if (roomDO.getRoomScoreEnabled() != null) {
                entity.setRoomScoreEnabled(roomDO.getRoomScoreEnabled());
            }
            if (roomDO.getRoomChatEnabled() != null) {
                entity.setRoomChatEnabled(roomDO.getRoomChatEnabled());
            }
            if (roomDO.getStatus() != null) {
                entity.setStatus(roomDO.getStatus());
            }
            if (roomDO.getFamilyId() != null) {
                entity.setFamilyId(roomDO.getFamilyId());
            }
            if (roomDO.getRoomCreateTime() != null) {
                entity.setRoomCreateTime(roomDO.getRoomCreateTime());
            }
            entity.setRoomUpdateTime(System.currentTimeMillis());
            RtcRoomEntity rtcRoomEsSave = rtcRoomRepository.save(entity);
            log.info("语聊房更新ES entity={}, result={}", JSON.toJSONString(entity), JSON.toJSONString(rtcRoomEsSave));
            return true;
        }
        log.info("对象不能为空");
        return false;
    }

    @Override
    public PageList<RtcRoomSearchItem> searchRtcRoom(RtcRoomSearchReq searchReq) {

        PageList<RtcRoomSearchItem> page = new PageList<>(searchReq.getPageIndex(), searchReq.getPageSize());
        if (searchReq != null && StringUtils.isNotEmpty(searchReq.getParams())) {
            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
            BoolQueryBuilder shouldQueryBuilder = QueryBuilders.boolQuery();
            // 搜索房间号、房间名
            shouldQueryBuilder.should(QueryBuilders.matchQuery("roomNo", searchReq.getParams()));
            shouldQueryBuilder.should(QueryBuilders.matchQuery("roomName", searchReq.getParams()));
            shouldQueryBuilder.should(QueryBuilders.wildcardQuery("roomName", String.format("*%s*", searchReq.getParams())));

            boolQueryBuilder.must(shouldQueryBuilder);

            log.info("查询语句：{}", boolQueryBuilder.toString());
            PageRequest pageRequest = PageRequest.of(searchReq.getPageIndex() - 1, searchReq.getPageSize(), Sort.by(Sort.Direction.DESC, "roomUpdateTime"));
            Page<RtcRoomEntity> searchResult = rtcRoomRepository.search(boolQueryBuilder, pageRequest);
            List<RtcRoomEntity> content = searchResult.getContent();
            log.info("es返回的结果内容：{}", JSON.toJSONString(content, true));

            page.setTotal(Long.valueOf(searchResult.getTotalElements()).intValue());
            List<RtcRoomSearchItem> items = new ArrayList<>();
            if (!CollectionUtils.isEmpty(content)) {
                List<Long> roomIds = content.stream().map(RtcRoomEntity::getRoomId).collect(Collectors.toList());
                List<Long> userIds = content.stream().map(RtcRoomEntity::getUserId).collect(Collectors.toList());
                List<Long> categoryIds = content.stream().map(RtcRoomEntity::getCategoryId).collect(Collectors.toList());
                // 房间=>热度
                Map<Long, Long> roomHeats = rtcRoomService.getRoomHeatMap(roomIds);
                // 是否pk中
                Map<Long, Boolean> statusMap = rtcBattleService.getRoomInBattleStatusMap(new HashSet<>(roomIds));
                // 用户
                List<UserDTO> users = userService.getUsers(userIds);
                // 分类
                List<RtcRoomCategory> roomCategories = rtcRoomCategoryDAO.selectCategories(categoryIds);
                for (RtcRoomEntity entity : content) {
                    RtcRoomSearchItem item = new RtcRoomSearchItem();
                    Long roomId = entity.getRoomId();
                    if (roomId == null) {
                        roomId = 0L;
                    }
                    item.setRoomId(roomId);
                    item.setUserId(entity.getUserId());
                    item.setUserNo(entity.getUserNo());
                    item.setRoomType(entity.getRoomType());
                    item.setRoomNo(entity.getRoomNo());
                    item.setRoomName(entity.getRoomName());
                    item.setRoomTitle(entity.getRoomTitle());
                    item.setRoomCover(entity.getRoomCover());
                    item.setStatus(entity.getStatus());
                    // 热度
                    Long roomHeat = 0L;
                    if (CollectionUtil.isNotEmpty(roomHeats)) {
                        Long tmp = roomHeats.get(roomId);
                        if (tmp != null) {
                            roomHeat = tmp;
                        }
                    }
                    item.setRoomHeat(roomHeat);
                    // pk状态
                    item.setInBattle(CollectionUtils.isEmpty(statusMap) ? BoolIntVal.False : statusMap.get(roomId) ? BoolIntVal.True : BoolIntVal.False);
                    item.setCategoryName(roomCategories.stream().filter(e -> e.getCategoryId().equals(entity.getCategoryId())).findFirst().map(RtcRoomCategory::getCategoryName).orElse(""));
                    item.setUserName(users.stream().filter(e -> e.getUserId().equals(entity.getUserId())).findFirst().map(UserDTO::getUserName).orElse(""));
                    items.add(item);
                }
            }
            page.setList(items);
        }
        return page;
    }

    /**
     * 同步所有语聊房间数据到es
     *
     * @return 同步数量
     */
    @Override
    public int syncAll2Es() {
        log.info("同步语聊房开始");
        Integer pageIndex = 1;
        // PageList 中最大为 50
        int pageSize = 50;

        // 同步语聊房数量
        AtomicInteger count = new AtomicInteger();
        while (pageIndex != null) {
            UserPageListQO qo = new UserPageListQO();
            qo.setPageIndex(pageIndex);
            qo.setPageSize(pageSize);
            PageList<RtcRoomListItem> pageList = rtcRoomService.getRoomList(pageIndex, pageSize, null);
            List<RtcRoomListItem> rooms = pageList.getList();
            if (CollectionUtils.isEmpty(rooms)) {
                break;
            }

            rooms.stream()
                    .filter(Objects::nonNull)
                    .filter(room -> room.getRoomId() != null)
                    .forEach(room -> {
                        RtcRoomDO roomDO = new RtcRoomDO();
                        BeanUtils.copyProperties(room, roomDO);
                        boolean result = saveOrUpdate(roomDO);
                        if (result) {
                            count.getAndIncrement();
                        }
                    });

            pageIndex = nextPage(pageList);
        }
        log.info("同步语聊房到ES结束，共 {} 条", count);

        return count.get();
    }
}
