package com.starbuds.server.service.data.impl;

import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.pojo.data.SkillVO;
import com.starbuds.server.common.pojo.db.SkillDO;
import com.starbuds.server.common.service.data.SkillService;
import com.starbuds.server.service.data.dao.FamilySkillDAO;
import com.starbuds.server.service.data.dao.SkillDAO;
import com.starbuds.server.service.data.dao.SkillTagDAO;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/10/15 17:27
 * @Description 技能业务
 */
@Service(version = "1.0.0")
public class SkillServiceImpl implements SkillService {

    @Resource
    SkillDAO skillDAO;

    @Resource
    FamilySkillDAO familySkillDAO;

    @Resource
    SkillTagDAO skillTagDAO;

    /**
     * 判断技能是否存在
     *
     * @param skillId
     * @return
     */
    @Override
    public boolean isSkillExist(Long skillId) {
        return skillDAO.countById(skillId) > 0;
    }

    /**
     * 判断技能是否存在
     *
     * @param familyId
     * @param userId
     * @param skillId
     * @return
     */
    @Override
    public boolean isSkillEnable(Long familyId, Long userId, Long skillId) {
        return skillDAO.countByIdAndStatus(skillId, CommonStatus.Normal) > 0 && familySkillDAO.countByIdAndStatus(familyId, userId, skillId, CommonStatus.Normal) > 0;
    }

    /**
     * 判断技能标签是否存在
     *
     * @param skillTagId
     * @return
     */
    @Override
    public boolean isSkillTagExist(Long skillTagId) {
        return skillTagDAO.countById(skillTagId) > 0;
    }

    /**
     * 获取技能列表
     *
     * @return
     */
    @Override
    public List<SkillVO> getSkills() {
        return skillDAO.selectSkills();
    }

    /**
     * 获取技能名称
     *
     * @param skillId
     * @return
     */
    @Override
    public String getSkillName(Long skillId) {
        return skillDAO.selectSkillName(skillId);
    }

    /**
     * 获取技能详情
     *
     * @param skillId
     * @return
     */
    @Override
    public SkillDO getSkillDetail(Long skillId) {
        return skillDAO.selectSkillDetail(skillId);
    }
}
