package com.starbuds.server.service.data.impl;

import com.starbuds.server.common.constant.message.sms.SmsActionEnum;
import com.starbuds.server.common.constant.message.sms.SmsTemplateTypeEnum;
import com.starbuds.server.common.constant.message.sms.SmsTypeEnum;
import com.starbuds.server.common.pojo.data.SmsAgentKeySecretName;
import com.starbuds.server.common.pojo.db.SmsAgentDO;
import com.starbuds.server.common.pojo.db.SmsSignDO;
import com.starbuds.server.common.service.data.SmsConfigService;
import com.starbuds.server.service.data.dao.SmsAgentDAO;
import com.starbuds.server.service.data.dao.SmsSignDAO;
import com.starbuds.server.service.data.dao.SmsTemplateDAO;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Optional;

/**
 * 短信配置服务实现
 *
 * @author Lay
 * @date 2019/12/6
 */
@Service(version = "1.0.0")
public class SmsConfigServiceImpl implements SmsConfigService {

    @Autowired
    SmsAgentDAO smsAgentDAO;

    @Autowired
    SmsSignDAO smsSignDAO;

    @Autowired
    SmsTemplateDAO smsTemplateDAO;

    /**
     * 获取服务商密钥对
     *
     * @param agentId
     * @return
     */
    @Override
    public SmsAgentKeySecretName getAgentKeyAndSecret(Long agentId) {
        SmsAgentDO smsAgentDO = smsAgentDAO.selectKeyAndSecretById(agentId);
        if (smsAgentDO != null) {
            SmsAgentKeySecretName smsAgentKeySecretName = new SmsAgentKeySecretName();
            smsAgentKeySecretName.setAccessKey(smsAgentDO.getAgentAccessKey());
            smsAgentKeySecretName.setAccessSecret(smsAgentDO.getAgentAccessSecret());
            smsAgentKeySecretName.setAgentName(smsAgentDO.getAgentName());
            return smsAgentKeySecretName;
        }
        return null;
    }

    /**
     * 获取短信签名
     *
     * @param signId
     * @param agentId optional
     * @return
     */
    @Override
    public SmsSignDO getSign(Long signId, Long agentId) {
        return smsSignDAO.selectSignById(signId, agentId);
    }

    /**
     * 获取短信模板代码
     *
     * @param agentId
     * @param smsTypeEnum
     * @param smsActionEnum
     * @param smsTemplateEnum
     * @return
     */
    @Override
    public String getTemplateCode(Long agentId, SmsTypeEnum smsTypeEnum, SmsActionEnum smsActionEnum, SmsTemplateTypeEnum smsTemplateEnum) {
        return smsTemplateDAO.selectCodeByAgentIdSmsTypeAndSmsAction(agentId,
                Optional.ofNullable(smsTypeEnum).map(t -> t.getCode()).orElse(null),
                Optional.ofNullable(smsActionEnum).map(t -> t.getCode()).orElse(null),
                Optional.ofNullable(smsTemplateEnum).map(t -> t.getCode()).orElse(null));
    }
}
