package com.starbuds.server.service.data.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.media.LiveStatusEnum;
import com.starbuds.server.common.constant.user.UserSortTypeEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.data.NearUserDTO;
import com.starbuds.server.common.pojo.data.NearbyUser;
import com.starbuds.server.common.pojo.data.UserSearchReq;
import com.starbuds.server.common.pojo.data.UserSearchResp;
import com.starbuds.server.common.pojo.db.UserAuthDO;
import com.starbuds.server.common.pojo.db.UserDO;
import com.starbuds.server.common.pojo.user.UserBasicInfo;
import com.starbuds.server.common.pojo.user.UserPageListQO;
import com.starbuds.server.common.pojo.user.UserSimpleInfo;
import com.starbuds.server.common.pojo.util.DistanceUtil;
import com.starbuds.server.common.service.data.AreaService;
import com.starbuds.server.common.service.data.UserSearchService;
import com.starbuds.server.common.service.user.SocialRelationService;
import com.starbuds.server.common.service.user.UserAuthService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.data.dao.UserDAO;
import com.starbuds.server.service.data.es.dao.UserRepository;
import com.starbuds.server.service.data.es.entity.UserEntity;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.elasticsearch.common.unit.DistanceUnit;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.sort.GeoDistanceSortBuilder;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.elasticsearch.core.geo.GeoPoint;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.scheduling.annotation.Async;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.awt.geom.Point2D;
import java.util.*;
import java.util.function.Function;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static com.starbuds.server.common.pojo.util.PageListUtil.nextPage;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/28 11:45
 * @Description 用户搜索接口实现
 */
@Slf4j
@Service(version = "1.0.0")
public class UserSearchServiceImpl implements UserSearchService {

    @Resource
    UserRepository userRepository;

    @Resource
    private UserDAO userDAO;

    @Resource
    private AreaService areaService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    @Reference(version = "1.0.0")
    private UserAuthService userAuthService;

    /**
     * 添加或更新用户
     *
     * @param userDO 用户
     * @return 是否成功
     */
    @Override
    public boolean insertOrUpdate(UserDO userDO) {
        if (userDO != null) {
            //待写入es的对象
            UserEntity entity = new UserEntity();
            //数据库获取用户基本信息
            UserBasicInfo dbUserInfo = userService.getBasicInfo(userDO.getUserId());
            //如果db存在
            if (dbUserInfo != null) {
                //给entity赋值
                BeanUtils.copyProperties(dbUserInfo, entity);
            }
            entity.setUserId(userDO.getUserId());

            //更新的字段以传入的为准
            if (StringUtils.isNotEmpty(userDO.getUserName())) {
                entity.setUserName(userDO.getUserName());
                entity.setUserNameIK(entity.getUserName());
            }
            if (userDO.getUserStatus() != null) {
                entity.setUserStatus(userDO.getUserStatus());
            }
            if (userDO.getUserNo() != null) {
                entity.setUserNo(userDO.getUserNo());
            }
            if (userDO.getNiceNo() != null) {
                entity.setNiceNo(userDO.getNiceNo());
            }
            if (StringUtils.isNotEmpty(userDO.getUserAvatar())) {
                entity.setUserAvatar(userDO.getUserAvatar());
            }
            if (userDO.getUserSex() != null) {
                entity.setUserSex(userDO.getUserSex());
            }
            if (userDO.getCountryId() != null) {
                entity.setCountryId(userDO.getCountryId());
            }
            if (userDO.getAreaId() != null) {
                entity.setAreaId(userDO.getAreaId());
            }
            if (userDO.getWealthLevel() != null) {
                entity.setWealthLevel(userDO.getWealthLevel());
            }
            if (userDO.getCharmLevel() != null) {
                entity.setCharmLevel(userDO.getCharmLevel());
            }
            if (userDO.getStarLevel() != null) {
                entity.setStarLevel(userDO.getStarLevel());
            }
            if (userDO.getChannelId() != null) {
                entity.setChannelId(userDO.getChannelId());
            }
            if (StringUtils.isNotEmpty(userDO.getUserPhone())) {
                entity.setUserPhone(userDO.getUserPhone());
            }
            if (userDO.getUserCreateTime() != null) {
                entity.setUserCreateTime(userDO.getUserCreateTime());
            }
            if (userDO.getUserUpdateTime() != null) {
                entity.setUserUpdateTime(userDO.getUserUpdateTime());
            }
            if (userDO.getIsAnchor() != null) {
                entity.setIsAnchor(userDO.getIsAnchor());
            }
            if (userDO.getUserAge() != null) {
                entity.setUserAge(userDO.getUserAge());
            }
            if (userDO.getIsOnline() != null) {
                entity.setIsOnline(userDO.getIsOnline());
            }
            if (userDO.getUserLatitude() != null && userDO.getUserLongitude() != null) {
                entity.setUserLatitude(userDO.getUserLatitude());
                entity.setUserLongitude(userDO.getUserLongitude());
                entity.setLocation(new GeoPoint(userDO.getUserLatitude(), userDO.getUserLongitude()));
            }
            userRepository.save(entity);
            return true;
        }
        log.info("对象不能为空");
        return false;
    }

    /**
     * APP搜索用户
     *
     * @param searchReq 查询条件
     * @return
     */
    @Override
    public PageList<UserSearchResp> searchUser(UserSearchReq searchReq) {
        PageList<UserSearchResp> page = new PageList<>(searchReq.getPageNum(), searchReq.getPageSize());
        if (searchReq != null && StringUtils.isNotEmpty(searchReq.getParams())) {
            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
            BoolQueryBuilder shouldQueryBuilder = QueryBuilders.boolQuery();
            //纯数字时查星芽号、靓号
            if (isNumber(searchReq.getParams())) {
                shouldQueryBuilder.should(QueryBuilders.matchQuery("userNo", searchReq.getParams()));
                shouldQueryBuilder.should(QueryBuilders.matchQuery("niceNo", searchReq.getParams()));
            }
            shouldQueryBuilder.should(QueryBuilders.matchQuery("userNameIK", searchReq.getParams()));
//            shouldQueryBuilder.should(QueryBuilders.wildcardQuery("userName", String.format("*%s*", searchReq.getParams())));

            boolQueryBuilder.must(shouldQueryBuilder);

//            PageRequest pageRequest = PageRequest.of(searchReq.getPageNum() - 1, searchReq.getPageSize(), Sort.by(Sort.Direction.DESC, "userUpdateTime"));
            PageRequest pageRequest = PageRequest.of(searchReq.getPageNum() - 1, searchReq.getPageSize());
            Page<UserEntity> searchResult = userRepository.search(boolQueryBuilder, pageRequest);
            List<UserEntity> content = searchResult.getContent();

            page.setTotal(Long.valueOf(searchResult.getTotalElements()).intValue());
            List<UserSearchResp> respList = new ArrayList<>();
            if (CollectionUtil.isNotEmpty(content)) {
                Long curUserId = searchReq.getCurUserId();
                Set<Long> userIds = content.stream().map(UserEntity::getUserId).collect(Collectors.toSet());
                // 关注map
                Map<Long, Boolean> followerMap = null;
                if (curUserId != null) {
                    followerMap = socialRelationService.isFollowers(curUserId, userIds);
                }
                for (UserEntity entity : content) {
                    UserSearchResp searchResp = new UserSearchResp();
                    searchResp.setUserId(entity.getUserId());
                    searchResp.setUserName(entity.getUserName());
                    searchResp.setUserNo(entity.getUserNo());
                    searchResp.setNiceNo(entity.getNiceNo());
                    searchResp.setUserAvatar(entity.getUserAvatar());
                    searchResp.setIsLive(userService.isLive(entity.getUserId()) ? BoolIntVal.True : BoolIntVal.False);
                    searchResp.setWealthLevel(entity.getWealthLevel());
                    searchResp.setCharmLevel(entity.getCharmLevel());
                    searchResp.setStarLevel(entity.getStarLevel());
                    searchResp.setUserLatitude(entity.getUserLatitude());
                    searchResp.setUserLongitude(entity.getUserLongitude());
                    Long userId = entity.getUserId();
                    if (curUserId != null) {
                        searchResp.setFollowStatus(CollectionUtil.isNotEmpty(followerMap) ? followerMap.get(userId) : false);
                    } else {
                        searchResp.setFollowStatus(false);
                    }
                    //距离
                    if (entity.getUserLatitude() != null && entity.getUserLongitude() != null) {
                        searchResp.setUserDistance(DistanceUtil.getDistancePretty(new Point2D.Double(entity.getUserLatitude(), entity.getUserLongitude()), new Point2D.Double()));
                    }
                    respList.add(searchResp);
                }
            }
            page.setList(respList);
        }
        return page;
    }


    /**
     * 获取用户id列表
     *
     * @param userNo    星芽号
     * @param niceNo    靓号
     * @param userPhone 用户手机号
     * @param isAnchor  是否主播
     * @param channelId 渠道id
     * @param pageNum   当前页码
     * @param pageSize  分页大小
     * @param sortType  排序方式
     * @param isAsc     是否升序
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return 分页对象
     */
    @Override
    public PageList<Long> getUserPageList(Long userNo, Long niceNo, String userName, Integer userStatus, String userPhone, String idCardNo,
                                          Integer isAnchor, Long channelId, int pageNum, int pageSize, int sortType, boolean isAsc, Long startTime, Long endTime) {
        PageList<Long> page = new PageList<>(pageNum, pageSize);
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();

        // 星芽号
        if (userNo != null) {
            boolQueryBuilder.must(QueryBuilders.matchQuery("userNo", userNo));
        }

        // 靓号
        if (niceNo != null) {
            boolQueryBuilder.must(QueryBuilders.matchQuery("niceNo", niceNo));
        }

        // 是否主播
        if (isAnchor != null) {
            boolQueryBuilder.must(QueryBuilders.matchQuery("isAnchor", isAnchor));
        }

        // 渠道id
        if (channelId != null) {
            boolQueryBuilder.must(QueryBuilders.matchQuery("channelId", channelId));
        }

        // 手机号
        if (StringUtils.isNotEmpty(userPhone)) {
            BoolQueryBuilder shouldQueryBuilder = QueryBuilders.boolQuery();
            shouldQueryBuilder.should(QueryBuilders.matchQuery("userPhone", userPhone));
            shouldQueryBuilder.should(QueryBuilders.wildcardQuery("userPhone", String.format("*%s*", userPhone)));
            boolQueryBuilder = boolQueryBuilder.must(shouldQueryBuilder);
        }

        // 用户姓名
        if (StringUtils.isNotEmpty(userName)) {
            BoolQueryBuilder shouldQueryBuilder = QueryBuilders.boolQuery();
            shouldQueryBuilder.should(QueryBuilders.matchQuery("userName", userName));
            shouldQueryBuilder.should(QueryBuilders.wildcardQuery("userName", String.format("*%s*", userName)));
            boolQueryBuilder = boolQueryBuilder.must(shouldQueryBuilder);
        }

        // 用户状态
        if (userStatus != null) {
            boolQueryBuilder.must(QueryBuilders.matchQuery("userStatus", userStatus));
        }

        // 注册时间
        if (startTime != null) {
            boolQueryBuilder.must(QueryBuilders.rangeQuery("userCreateTime").gte(startTime));
        }

        // 注册时间
        if (endTime != null) {
            boolQueryBuilder.must(QueryBuilders.rangeQuery("userCreateTime").lte(endTime));
        }

        if (StringUtils.isNotBlank(idCardNo)) {
            boolQueryBuilder.must(QueryBuilders.matchQuery("idCardNo", idCardNo));
        }

        // 排序方式
        UserSortTypeEnum userSortType = UserSortTypeEnum.toEnum(sortType);
        // 默认值
        if (userSortType == null) {
            userSortType = UserSortTypeEnum.USER_CREATE_TIME;
        }

        NativeSearchQueryBuilder queryBuilder = new NativeSearchQueryBuilder();
        queryBuilder.withQuery(boolQueryBuilder);
        // 排序
        queryBuilder.withSort(SortBuilders.fieldSort(userSortType.getOrderFiled()).order(isAsc ? SortOrder.ASC : SortOrder.DESC));
        // 分页
        queryBuilder.withPageable(PageRequest.of(pageNum - 1, pageSize));

        Page<UserEntity> searchResult = userRepository.search(queryBuilder.build());

        List<UserEntity> content = searchResult.getContent();

        page.setTotal(Long.valueOf(searchResult.getTotalElements()).intValue());

        List<Long> userIds = new ArrayList<>();
        if (!CollectionUtils.isEmpty(content)) {
            for (UserEntity entity : content) {
                userIds.add(entity.getUserId());
            }
        }
        page.setList(userIds);
        return page;
    }

    /**
     * 获取用户列表
     *
     * @param startTime
     * @param endTime
     * @return
     */
    @Override
    public List<Long> getUserListByCreateTime(Long startTime, Long endTime) {
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();

        // 注册时间
        if (startTime != null) {
            boolQueryBuilder.must(QueryBuilders.rangeQuery("userCreateTime").gte(startTime));
        }

        // 注册时间
        if (endTime != null) {
            boolQueryBuilder.must(QueryBuilders.rangeQuery("userCreateTime").lte(endTime));
        }

        NativeSearchQueryBuilder queryBuilder = new NativeSearchQueryBuilder();
        queryBuilder.withQuery(boolQueryBuilder);
        // 排序
        queryBuilder.withSort(SortBuilders.fieldSort("userCreateTime").order(SortOrder.DESC));

        Iterator<UserEntity> iterator = userRepository.search(queryBuilder.build()).iterator();
        List<Long> userIds = new ArrayList<>();
        while (iterator.hasNext()) {
            userIds.add(iterator.next().getUserId());
        }
        return userIds;
    }

    /**
     * 删除用户
     *
     * @param userId 用户id
     * @return 是否成功
     */
    @Override
    public boolean deleteById(Long userId) {
        userRepository.deleteById(userId);
        return true;
    }

    /**
     * 同步用户es数据
     *
     * @param userIds 用户id
     * @return 是否成功
     */
    @Override
    public boolean syncUser(List<Long> userIds) {
        if (!CollectionUtils.isEmpty(userIds)) {
            Map<Long, UserAuthDO> authMap = userAuthService.selectAuthMapByIds(userIds);

            for (Long userId : userIds) {
                UserDO userDO = userService.selectById(userId);
                UserEntity entity = new UserEntity();
                if (userDO != null) {
                    BeanUtils.copyProperties(userDO, entity);
                    userRepository.deleteById(userId);
                    // 地理位置
                    if (entity.getUserLatitude() != null && entity.getUserLongitude() != null) {
                        entity.setLocation(new GeoPoint(entity.getUserLatitude(), entity.getUserLongitude()));
                    }
                    entity.setUserNameIK(entity.getUserName());

                    // 身份证号
                    UserAuthDO userAuth = authMap.get(userId);
                    if (userAuth != null && StringUtils.isNotBlank(userAuth.getIdCardNo())) {
                        entity.setIdCardNo(userAuth.getIdCardNo());
                    }
                    userRepository.save(entity);
                }
            }
            return true;
        }
        return false;
    }

    /**
     * 同步所有用户数据到es
     *
     * @return true 成功
     */
    @Override
    @Async
    public void syncAllUser() {
        log.info("同步用户到ES 开始");
        Integer pageIndex = 1;
        // PageList 中最大为 50
        int pageSize = 50;

        // 同步用户数量
        int count = 0;
        while (pageIndex != null) {
            UserPageListQO qo = new UserPageListQO();
            qo.setPageIndex(pageIndex);
            qo.setPageSize(pageSize);
            PageList<UserDO> pageList = userService.getUserPageList(qo);
            List<UserDO> users = pageList.getList();
            if (CollectionUtils.isEmpty(users)) {
                break;
            }

            List<Long> userIds = users.stream().map(UserDO::getUserId).collect(Collectors.toList());
            Map<Long, UserAuthDO> authMap = userAuthService.selectAuthMapByIds(userIds);

            List<UserEntity> entities = new ArrayList<>(users.size());
            users.stream()
                    .filter(Objects::nonNull)
                    .filter(user -> user.getUserId() != null)
                    .forEach(user -> {
                        UserEntity entity = new UserEntity();
                        BeanUtils.copyProperties(user, entity);
                        // 地理位置
                        if (entity.getUserLatitude() != null && entity.getUserLongitude() != null) {
                            entity.setLocation(new GeoPoint(entity.getUserLatitude(), entity.getUserLongitude()));
                        }
                        entity.setUserNameIK(entity.getUserName());

                        // 身份证号
                        UserAuthDO userAuth = authMap.get(user.getUserId());
                        if (userAuth != null && StringUtils.isNotBlank(userAuth.getIdCardNo())) {
                            entity.setIdCardNo(userAuth.getIdCardNo());
                        }

                        entities.add(entity);
                    });

            userRepository.saveAll(entities);
            count += entities.size();
            pageIndex = nextPage(pageList);
        }
        log.info("同步用户到ES {} 条", count);

//        return count;
    }

    /**
     * 同步所有用户的定位城市
     *
     * @return
     */
    @Override
    @Async
    public void syncAllUserLocationCity() {
        log.info("同步所有用户的定位城市开始");
        Integer pageIndex = 1;
        // PageList 中最大为 50
        int pageSize = 50;

        // 同步用户数量
        int count = 0;
        while (pageIndex != null) {
            UserPageListQO qo = new UserPageListQO();
            qo.setPageIndex(pageIndex);
            qo.setPageSize(pageSize);
            PageList<UserDO> pageList = userService.getUserPageList(qo);
            List<UserDO> users = pageList.getList();
            if (CollectionUtils.isEmpty(users)) {
                break;
            }

            count += users.parallelStream()
                    .filter(Objects::nonNull)
                    .filter(user -> user.getUserId() != null)
                    .filter(user -> user.getUserLongitude() != null)
                    .filter(user -> user.getUserLatitude() != null)
                    .filter(user -> StringUtils.isBlank(user.getLocationCity()))
                    .mapToInt(user -> userDAO.updateLocationCity(user.getUserId(),
                            areaService.getLocationCity(user.getUserLongitude(), user.getUserLatitude())))
                    .sum();
            log.info("已更新 {} 条", count);
            pageIndex = nextPage(pageList);
        }
        log.info("同步所有用户的定位城市 {} 条", count);

//        return count;
    }

    /**
     * 更新直播直播状态
     *
     * @param userId
     * @param liveStatusEnum
     * @return
     */
    @Override
    public boolean setLiveStatus(Long userId, LiveStatusEnum liveStatusEnum) {
        UserEntity entity = userRepository.findById(userId).orElse(null);
        if (entity == null) {
            return false;
        }
        entity.setLiveStatus(liveStatusEnum.getValue());
        entity.setUserUpdateTime(System.currentTimeMillis());
        userRepository.save(entity);
        return true;
    }

    /**
     * 设置靓号
     *
     * @param userId
     * @param niceNo
     * @return
     */
    @Override
    public boolean setNiceNo(Long userId, Long niceNo) {
        UserEntity entity = userRepository.findById(userId).orElse(null);
        if (entity == null) {
            return false;
        }
        entity.setNiceNo(niceNo);
        entity.setUserUpdateTime(System.currentTimeMillis());
        userRepository.save(entity);
        return true;
    }

    /**
     * 设置单聊权限
     *
     * @param userId
     * @param singleChatEnable
     * @param singleChatCategoryId
     * @return
     */
    @Override
    public boolean setSingleChat(Long userId, Integer singleChatEnable, Long singleChatCategoryId) {
        UserEntity entity = userRepository.findById(userId).orElse(null);
        if (entity == null) {
            return false;
        }
        entity.setSingleChatEnable(singleChatEnable);
        entity.setSingleChatCategoryId(singleChatCategoryId);
        entity.setUserUpdateTime(System.currentTimeMillis());
        userRepository.save(entity);
        return true;
    }

    @Override
    public boolean setOnlineStatus(Long userId, Integer isOnline) {
        UserEntity entity = userRepository.findById(userId).orElse(null);
        if (entity == null) {
            return false;
        }
        entity.setIsOnline(isOnline);
        entity.setUserUpdateTime(System.currentTimeMillis());
        userRepository.save(entity);
        return true;
    }

    /**
     * 设置用户状态
     *
     * @param userIds
     * @param userStatus
     * @return
     */
    @Override
    public boolean setUserStatus(List<Long> userIds, Integer userStatus) {
        if (CollectionUtil.isEmpty(userIds)) {
            return false;
        }
        long curTime = System.currentTimeMillis();
        for (Long userId : userIds) {
            UserEntity entity = userRepository.findById(userId).orElse(null);
            if (entity == null) {
                continue;
            }
            entity.setUserStatus(userStatus);
            entity.setUserUpdateTime(curTime);
            userRepository.save(entity);
        }
        return true;
    }

    /**
     * 设置用户位置
     *
     * @param userId
     * @param userLatitude
     * @param userLongitude
     * @param areaId
     * @param locationCity  定位城市
     * @return
     */
    @Override
    public boolean setPosition(Long userId, Double userLatitude, Double userLongitude, Integer areaId, String locationCity) {
        UserEntity entity = userRepository.findById(userId).orElse(null);
        if (entity == null) {
            return false;
        }
        // 地理位置
        if (entity.getUserLatitude() != null && entity.getUserLongitude() != null) {
            entity.setUserLatitude(userLatitude);
            entity.setUserLongitude(userLongitude);
            entity.setLocation(new GeoPoint(entity.getUserLatitude(), entity.getUserLongitude()));
        }
        if (areaId != null) {
            entity.setAreaId(areaId);
        }
        if (locationCity != null) {
            entity.setLocationCity(locationCity);
        }
        if (userLatitude != null || userLongitude != null || areaId != null) {
            entity.setUserUpdateTime(System.currentTimeMillis());
            userRepository.save(entity);
        }
        return true;
    }

    /**
     * 设置用户姓名
     *
     * @param userId
     * @param userName
     * @return
     */
    @Override
    public boolean setUserName(Long userId, String userName) {
        UserEntity entity = userRepository.findById(userId).orElse(null);
        if (entity == null) {
            return false;
        }
        if (StringUtils.isNotEmpty(userName)) {
            entity.setUserName(userName);
            entity.setUserNameIK(entity.getUserName());
            entity.setUserUpdateTime(System.currentTimeMillis());
            userRepository.save(entity);
        }
        return true;
    }

    @Override
    public boolean setWealthLevel(Long userId, Integer newLevel) {
        UserEntity entity = userRepository.findById(userId).orElse(null);
        if (entity == null) {
            return false;
        }
        entity.setWealthLevel(newLevel);
        entity.setUserUpdateTime(System.currentTimeMillis());
        userRepository.save(entity);
        return true;
    }

    @Override
    public boolean setCharmLevel(Long userId, Integer newLevel) {
        UserEntity entity = userRepository.findById(userId).orElse(null);
        if (entity == null) {
            return false;
        }
        entity.setCharmLevel(newLevel);
        entity.setUserUpdateTime(System.currentTimeMillis());
        userRepository.save(entity);
        return true;
    }

    @Override
    public boolean setStarLevel(Long userId, Integer newLevel) {
        UserEntity entity = userRepository.findById(userId).orElse(null);
        if (entity == null) {
            return false;
        }
        entity.setStarLevel(newLevel);
        entity.setUserUpdateTime(System.currentTimeMillis());
        userRepository.save(entity);
        return true;
    }

    /**
     * 获取附近的人
     *
     * @param userDTO
     * @return
     */
    @Override
    public PageList<NearbyUser> getNearUserPageList(NearUserDTO userDTO) {
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        // 区域id
        if (userDTO.getAreaId() != null) {
            boolQueryBuilder.must(QueryBuilders.matchQuery("areaId", userDTO.getAreaId()));
        }

        // 性别
        if (userDTO.getUserSex() != null) {
            boolQueryBuilder.must(QueryBuilders.matchQuery("userSex", userDTO.getUserSex()));
        }

        // 是否主播
        if (userDTO.getIsAnchor() != null) {
            boolQueryBuilder.must(QueryBuilders.matchQuery("isAnchor", userDTO.getIsAnchor()));
        }

        // 直播状态
        if (userDTO.getLiveStatusEnum() != null) {
            boolQueryBuilder.must(QueryBuilders.matchQuery("liveStatus", userDTO.getLiveStatusEnum().getValue()));
        }

        // 单聊权限
        if (userDTO.getSingleChatEnable() != null) {
            boolQueryBuilder.must(QueryBuilders.matchQuery("singleChatEnable", userDTO.getSingleChatEnable()));
        }

        // 是否在线
        if (userDTO.getIsOnline() != null) {
            boolQueryBuilder.must(QueryBuilders.matchQuery("isOnline", userDTO.getIsOnline()));
        }

        // 单聊分类
        if (userDTO.getSingleChatCategoryId() != null) {
            boolQueryBuilder.must(QueryBuilders.matchQuery("singleChatCategoryId", userDTO.getSingleChatCategoryId()));
        }

        // 自己不显示在附近的人列表中
        if (userDTO.getMyselfUserId() != null) {
            boolQueryBuilder.mustNot(QueryBuilders.matchQuery("userId", userDTO.getMyselfUserId()));
        }

        // 年龄
        if (userDTO.getUserAge() != null) {
            boolQueryBuilder.must(QueryBuilders.rangeQuery("userAge").lte(userDTO.getUserAge()));
        }
        if (userDTO.getUserAgeMin() != null) {
            boolQueryBuilder.must(QueryBuilders.rangeQuery("userAge").gte(userDTO.getUserAgeMin()));
        }
        if (userDTO.getUserAgeMax() != null) {
            boolQueryBuilder.must(QueryBuilders.rangeQuery("userAge").lte(userDTO.getUserAgeMax()));
        }

        NativeSearchQueryBuilder queryBuilder = new NativeSearchQueryBuilder();

        if (userDTO.getUserLatitude() != null && userDTO.getUserLongitude() != null && userDTO.getDistance() != null) {

            boolQueryBuilder.must(QueryBuilders.geoDistanceQuery("location").point(userDTO.getUserLatitude(), userDTO.getUserLongitude())
                    .distance(userDTO.getDistance(), DistanceUnit.METERS));
            queryBuilder.withQuery(boolQueryBuilder);

            if (BoolIntVal.True != userDTO.getIsNew()) {
                // 按距离升序排列
                GeoDistanceSortBuilder distanceSortBuilder = new GeoDistanceSortBuilder("location", userDTO.getUserLatitude(), userDTO.getUserLongitude());
                distanceSortBuilder.unit(DistanceUnit.METERS);
                distanceSortBuilder.order(SortOrder.ASC);
                queryBuilder.withSort(distanceSortBuilder);
            } else {
                queryBuilder.withSort(SortBuilders.fieldSort("userCreateTime").order(SortOrder.DESC));
            }
        } else {
            queryBuilder.withSort(SortBuilders.fieldSort("userCreateTime").order(SortOrder.DESC));
        }

        // 分页
        queryBuilder.withPageable(PageRequest.of(userDTO.getPageIndex() - 1, userDTO.getPageSize()));
        Page<UserEntity> searchResult = userRepository.search(queryBuilder.build());
        PageList<NearbyUser> page = new PageList<>(userDTO.getPageIndex(), userDTO.getPageSize());
        List<NearbyUser> list = new ArrayList<>();
        Point2D.Double userPoint = new Point2D.Double(userDTO.getUserLatitude(), userDTO.getUserLongitude());
        for (Iterator<UserEntity> iterator = searchResult.iterator(); iterator.hasNext(); ) {
            UserEntity userEntity = iterator.next();
            NearbyUser nearbyUser = new NearbyUser();
            nearbyUser.setUserId(userEntity.getUserId());
            nearbyUser.setLatitude(userEntity.getUserLatitude() == null ? 0 : userEntity.getUserLatitude());
            nearbyUser.setLongitude(userEntity.getUserLongitude() == null ? 0 : userEntity.getUserLongitude());
            // 计算距离
            nearbyUser.setDistance(DistanceUtil.getDistancePretty(userPoint, new Point2D.Double(nearbyUser.getLatitude(), nearbyUser.getLongitude())));
            nearbyUser.setLocationCity(userEntity.getLocationCity());
            list.add(nearbyUser);
        }

        page.setList(list);
        page.setTotal(Long.valueOf(searchResult.getTotalElements()).intValue());
        return page;
    }

    @Override
    public PageList<Long> getAnchorList(String content, int pageIndex, int pageSize) {
        PageList<Long> page = new PageList<>(pageIndex, pageSize);
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        BoolQueryBuilder shouldQueryBuilder = QueryBuilders.boolQuery();

        boolQueryBuilder.must(QueryBuilders.matchQuery("isAnchor", BoolIntVal.True));

        if (StringUtils.isNotEmpty(content)) {

            shouldQueryBuilder.should(QueryBuilders.matchQuery("userName", content));
            shouldQueryBuilder.should(QueryBuilders.wildcardQuery("userName", String.format("*%s*", content)));

            if (isNumber(content)) {
                shouldQueryBuilder.should(QueryBuilders.matchQuery("userNo", content));
            }
        }

        boolQueryBuilder.must(shouldQueryBuilder);

        NativeSearchQueryBuilder queryBuilder = new NativeSearchQueryBuilder();
        queryBuilder.withQuery(boolQueryBuilder);
        // 排序
        queryBuilder.withSort(SortBuilders.fieldSort("liveStatus").order(SortOrder.ASC));
        // 分页
        queryBuilder.withPageable(PageRequest.of(pageIndex - 1, pageSize));

        Page<UserEntity> searchResult = userRepository.search(queryBuilder.build());

        List<UserEntity> result = searchResult.getContent();

        List<Long> userIds = new ArrayList<>();
        if (!CollectionUtils.isEmpty(result)) {
            for (UserEntity entity : result) {
                userIds.add(entity.getUserId());
            }
        }
        page.setList(userIds);
        page.setTotal(Long.valueOf(searchResult.getTotalElements()).intValue());
        return page;
    }

    /**
     * 获取简单信息
     *
     * @param userIds
     * @return
     */
    @Override
    public Map<Long, UserSimpleInfo> getSimpleInfo(List<Long> userIds) {
        if (!CollectionUtil.isEmpty(userIds)) {
            List<UserSimpleInfo> list = userDAO.selectSimleInfoByIds(userIds);
            if (!CollectionUtils.isEmpty(list)) {
                return list.stream().collect(Collectors.toMap(UserSimpleInfo::getUserId, Function.identity()));
            }
        }

        return new HashMap<>();
    }

    /**
     * 判断是否纯数字
     *
     * @param str
     * @return
     */
    public static boolean isNumber(String str) {
        Pattern pattern = Pattern.compile("[0-9]*");
        return pattern.matcher(str).matches();
    }
}
