package com.starbuds.server.service.data.provider;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLDecoder;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/5/12 11:23
 * @Description 百度服务提供者
 */
@Slf4j
@Component
public class BaiduProvider {

    private static final Integer RETRY_TIMES = 3;

    /**
     * 向百度回传app激活转化数据
     * callbackFunc 调用callback url
     *
     * @param callbackUrl 监测地址中callback_url参数值
     * @param akey        当前账户对应akey
     * @return 调用成功返回true
     */
    public static boolean callback(String callbackUrl, String akey, String atype, String avalue) {
        String url = null;
        CloseableHttpClient client = HttpClients.createDefault();
        HttpGet httpGet = new HttpGet();
        try {
            url = URLDecoder.decode(callbackUrl, "UTF-8");
            // 注意：示例为app激活转化类型需要替换的参数，当想要优化其他类型时，请按照文档要求替换
            url = url.replace("{{ATYPE}}", atype);
            url = url.replace("{{AVALUE}}", avalue);
            // 计算sign值
            String sign = md5Func(url + akey);
            // 得到最终callback url
            url += "&sign=" + sign;
            httpGet.setURI(new URI(url));
            // 打印要调用的callbackrl
            log.info(url);
        } catch (UnsupportedEncodingException | URISyntaxException e) {
            e.printStackTrace();
            return false;
        }
        // 调用callback url，判断返回结果，添加重试次数3次
        for (int i = 0; i < RETRY_TIMES; i++) {
            try {
                HttpResponse httpResponse = client.execute(httpGet);
                int statusCode = httpResponse.getStatusLine().getStatusCode();
                if (statusCode == 200) {
                    String resp = EntityUtils.toString(httpResponse.getEntity());
                    log.info("retry times: " + i + " res: " + resp);
                    JsonObject jsonResp = new JsonParser().parse(resp).getAsJsonObject();
                    int code = jsonResp.get("error_code").getAsInt();
                    // error_code=500时为服务异常，可添加重试
                    if (code != 500) {
                        return code == 0; // 返回error_code为0时接口调用成功，否则需排查具体原因
                    }
                }
            } catch (IOException e) {
                log.info("调用出错{}", e);
            }
        }
        return false;
    }

    /**
     * 字符串md5
     *
     * @param string 需要md5加密的字符串
     * @return md5结果
     */
    private static String md5Func(String string) {
        byte[] hash;
        try {
            hash = MessageDigest.getInstance("MD5").digest(string.getBytes("UTF-8"));
        } catch (NoSuchAlgorithmException | UnsupportedEncodingException e) {
            return "";
        }
        StringBuilder hex = new StringBuilder(hash.length * 2);
        for (byte b : hash) {
            int i = (b & 0xFF);
            if (i < 0x10) {
                hex.append('0');
            }
            hex.append(Integer.toHexString(i));
        }
        return hex.toString();
    }

    public static void main(String[] args) {
        // 需替换为推广账户akey
        String akey = "MzA0NjI1NDI=";
        // 需替换为从监测url中解析出来的callback url
        String callbackUrl = "http%3A%2F%2Focpc.baidu.com%2Focpcapi%2Fcb%2FactionCb%3Fa_type%3D%7B%7BATYPE%7D%7D%26a_value%3D%7B%7BAVALUE%7D%7D%26s%3D99329%26ext_info%3DHhGPIYdRNg-xPWnsnHRzg1T3rHnsrHcvg1czndtsg10snRNjrjnLn1D1n1FawWDzwHKDnbmdPbmsPbRdP1TLgdtkPHRdPHDvP1b3";
        System.out.println(BaiduProvider.callback(callbackUrl, akey, "activate", "0"));
    }

}
