package com.starbuds.server.service.data.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.db.DataGreenUserDO;
import com.starbuds.server.service.data.dao.DataGreenUserDAO;
import com.starbuds.server.common.pojo.data.qo.DataGreenUserPageListQO;

import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 内容安全-用户信息
 */
@Component
@Slf4j
public class DataGreenUserProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "dataId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private DataGreenUserDAO dataGreenUserDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public DataGreenUserDO create(DataGreenUserDO po) {
        po.setDataId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        po.setIsDeleted(BoolIntVal.False);
        boolean flag = dataGreenUserDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("内容安全-用户信息创建成功 {}", json);
            return po;
        }
        log.warn("内容安全-用户信息创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<DataGreenUserDO> list) {
        boolean flag = dataGreenUserDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(DataGreenUserDO po) {
        boolean flag = dataGreenUserDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(DataGreenUserDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getDataId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = dataGreenUserDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public DataGreenUserDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return dataGreenUserDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return dataGreenUserDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<DataGreenUserDO> getPageList(DataGreenUserPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<DataGreenUserDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(dataGreenUserDAO.selectPageList(qw));
        page.setTotal(dataGreenUserDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, DataGreenUserDO> getMapByIds(Collection ids) {
        return dataGreenUserDAO.selectMapByIds(ids);
    }

    // 手动编码开始

}