package com.starbuds.server.service.data.provider;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyuncs.AcsResponse;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.RpcAcsRequest;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.exceptions.ServerException;
import com.aliyuncs.imageaudit.model.v20191230.ScanImageRequest;
import com.aliyuncs.imageaudit.model.v20191230.ScanImageResponse;
import com.starbuds.server.common.constant.data.ImageGreenAuditEnum;
import com.starbuds.server.common.pojo.data.GreenAuditResult;
import com.starbuds.server.common.pojo.data.ImageGreenAuditResult;
import com.starbuds.server.common.pojo.db.DataGreenDO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.service.data.config.GreenConfig;
import com.starbuds.server.service.data.dao.DataGreenDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 同步图片风险检测(阿里)
 *
 * @author aoe
 * @date 2020/5/28
 * @see AbstractGreenProvider 代替
 */
@Slf4j
@Component
@Deprecated
public class ImageGreenProvider {

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private GreenConfig greenConfig;

    @Resource
    private DataGreenDAO dataGreenDAO;

    private Map<String, String> suggestionMap;

    @PostConstruct
    private void init() {
        suggestionMap = new HashMap<>();
        suggestionMap.put("review_porn", "图片可能涉黄");
        suggestionMap.put("review_terrorism", "图片可能涉恐涉政");
        suggestionMap.put("review_ad", "图片可能包含垃圾广告");
        suggestionMap.put("review_logo", "图片可能包含logo");
        suggestionMap.put("block_porn", "图片涉黄");
        suggestionMap.put("block_terrorism", "图片涉恐涉政");
        suggestionMap.put("block_ad", "图片包含垃圾广告");
        suggestionMap.put("block_logo", "图片包含logo");
    }

    /**
     * 图片内容检测
     *
     * @param url    图片地址
     * @param dataId 鉴定业务Id
     * @return true 内容合规
     */
    public ImageGreenAuditResult scanImage(String url, long dataId) {
        ScanImageRequest req = new ScanImageRequest();
        req.setScenes(greenConfig.getGreenScenesList());
        List<ScanImageRequest.Task> tasks = new ArrayList<>();
        ScanImageRequest.Task task = new ScanImageRequest.Task();
        task.setDataId("" + dataId);
        task.setImageURL(url);
        tasks.add(task);
        req.setTasks(tasks);

        ImageGreenAuditResult auditResult = new ImageGreenAuditResult();
        auditResult.setResult(ImageGreenAuditEnum.REVIEW);
        ScanImageResponse resp;
        try {
            resp = getAcsResponse(req);
        } catch (Exception e) {
            log.error(String.format("图片内容审核请求超时，鉴定结果失败！ url=%s", url), e);
            auditResult.setErrMsg("图片内容审核请求超时，鉴定结果失败！");
            return auditResult;
        }
        if (resp == null) {
            log.warn(String.format("图片内容审核返回结果为 null，鉴定结果失败！ url=%s", url));
            auditResult.setErrMsg("图片内容审核返回结果为 null，鉴定结果失败！");
            return auditResult;
        }

        String res = JSON.toJSONString(resp);
        log.info("图片内容审核返回结果 url={} res={}", url, res);

        List<GreenAuditResult> auditResults = getAuditResult(res);
        if (CollectionUtils.isEmpty(auditResults)) {
            log.warn("图片内容审核返回结果为空，无法判断，返回鉴定失败！url={} dataId={}", url, dataId);
            auditResult.setErrMsg("图片内容审核返回结果为空，无法判断，返回鉴定失败！");
            return auditResult;
        }
        auditResult.setDetail(auditResults);
        saveRes(dataId, res, auditResults);

        // 优先返回拒绝的
        for (GreenAuditResult audit : auditResults) {
            String suggestion = audit.getSuggestion();
            if (StringUtils.isEmpty(suggestion)) {
                continue;
            }
            if ("block".equals(suggestion)) {
                auditResult.setResult(ImageGreenAuditEnum.BLOCK);
                auditResult.setSuggestion(getSuggestionMsg(suggestion, audit.getScene()));
                return auditResult;
            }
        }
        // 检测结果不确定
        for (GreenAuditResult audit : auditResults) {
            String suggestion = audit.getSuggestion();
            if (StringUtils.isEmpty(suggestion)) {
                continue;
            }
            if ("review".equals(suggestion)) {
                auditResult.setSuggestion(getSuggestionMsg(suggestion, audit.getScene()));
                return auditResult;
            }
        }

        auditResult.setResult(ImageGreenAuditEnum.PASS);
        auditResult.setSuggestion("通过");
        return auditResult;
    }

    private String getSuggestionMsg(String suggestion, String scene) {
        String msg = suggestionMap.get(suggestion + "_" + scene);
        return msg == null ? "审核结果不确定" : msg;
    }

    private <R extends RpcAcsRequest<T>, T extends AcsResponse> T getAcsResponse(R req) throws Exception {
        IAcsClient client = new DefaultAcsClient(greenConfig.getGreenProfile());
        try {
            return client.getAcsResponse(req);
        } catch (ServerException e) {
            // 服务端异常
            log.error(String.format("ServerException: errCode=%s, errMsg=%s", e.getErrCode(), e.getErrMsg()), e);
            throw e;
        } catch (ClientException e) {
            // 客户端错误
            log.error(String.format("ClientException: errCode=%s, errMsg=%s", e.getErrCode(), e.getErrMsg()), e);
            throw e;
        } catch (Exception e) {
            log.error("图片内容审核失败", e);
            throw e;
        }
    }

    private void saveRes(long dataId, String res, List<GreenAuditResult> auditResults) {
        DataGreenDO feedGreen = new DataGreenDO();
        feedGreen.setGreenId(identityService.getId());
        feedGreen.setDataId(dataId);
        feedGreen.setResult(res);
        feedGreen.setSuggestion(JSON.toJSONString(auditResults));
        feedGreen.setCreateTime(System.currentTimeMillis());
        int result = dataGreenDAO.insert(feedGreen);
        log.info("存储图片内容审核结果 result={} res={}", result, res);
    }

    /**
     * 获得审核结果
     *
     * @param res
     * @return
     */
    private List<GreenAuditResult> getAuditResult(String res) {
        List<GreenAuditResult> resultList = new ArrayList<>();
        JSONObject json = JSONObject.parseObject(res);
        JSONObject data = json.getJSONObject("data");
        if (data == null) {
            log.error("图片内容审核失败 data 为 null");
            return resultList;
        }
        JSONArray results = data.getJSONArray("results");
        if (results == null || results.size() < 1) {
            log.error("图片内容审核失败 results 为空 results={}", results);
            return resultList;
        }
        JSONObject result = (JSONObject) results.get(0);
        if (result == null) {
            log.error("图片内容审核失败 result 为 null");
            return resultList;
        }
        JSONArray subResults = result.getJSONArray("subResults");
        if (subResults == null || subResults.size() < 1) {
            log.error("图片内容审核失败 subResults 为空 subResults={}", results);
            return resultList;
        }
        for (int i = 0; i < subResults.size(); i++) {
            JSONObject sub = subResults.getJSONObject(i);
            if (sub == null) {
                continue;
            }
            GreenAuditResult audit = new GreenAuditResult();
            audit.setScene(sub.getString("scene"));
            audit.setRate(sub.getString("rate"));
            audit.setSuggestion(sub.getString("suggestion"));
            resultList.add(audit);
        }

        return resultList;
    }
}
