package com.starbuds.server.service.data.provider;

import cn.hutool.http.Header;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.finance.OcrStatEnum;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/4/29 13:10
 * @Description ocr识别(阿里)
 * @see OcrBaiduProvider 代替
 */
@Slf4j
@Component
@Deprecated
public class OcrProvider {

    @Value("${aliyun.ocr.appCode}")
    private String appCode;

    public static final String url = "https://dm-51.data.aliyun.com/rest/160601/ocr/ocr_idcard.json";

    public JSONObject doOcr(String imgUrl) {
        if (StringUtils.isEmpty(imgUrl)) {
            return new JSONObject();
        }
        JSONObject configObj = new JSONObject();
        configObj.put("side", "face");
        String config_str = configObj.toString();
        // 拼装请求body的json字符串
        JSONObject requestObj = new JSONObject();
        try {
            requestObj.put("image", getBase64(imgUrl));
            if (config_str.length() > 0) {
                requestObj.put("configure", config_str);
            }
            HttpResponse response = HttpRequest.post(url)
                    .header(Header.CONTENT_TYPE, "application/json;charset=utf-8;")
                    .header("Authorization", "APPCODE " + appCode)
                    .body(requestObj.toString())
                    .execute();
            int stat = response.getStatus();
            JSONObject result = new JSONObject();
            if (stat != 200) {
                String header = response.header("X-Ca-Error-Message");
                log.info("错误信息：{}", header);
                result.put("success", false);
                result.put("code", stat);
                result.put("errorMsg", OcrStatEnum.toEnum(stat).getErrorMsg());
                return result;
            }
            result = JSON.parseObject(response.body());
            log.info("返回结果：{}", result.toJSONString());
            return result;
        } catch (Exception e) {
            log.info("调用ocr失败", e);
        }
        return null;
    }

    /**
     * 对path进行判断，如果是本地文件就二进制读取并base64编码，如果是url,则返回
     *
     * @param path
     * @return
     */
    public static String getBase64(String path) {
        /**
         *  对path进行判断，如果是本地文件就二进制读取并base64编码，如果是url,则返回
         */
        String imgBase64 = "";
        if (path.startsWith("http")) {
            imgBase64 = path;
        } else {
            try {
                File file = new File(path);
                byte[] content = new byte[(int) file.length()];
                FileInputStream finputstream = new FileInputStream(file);
                finputstream.read(content);
                finputstream.close();
                imgBase64 = new String(Base64.encodeBase64(content));
            } catch (IOException e) {
                e.printStackTrace();
                return imgBase64;
            }
        }

        return imgBase64;
    }
}

