package com.starbuds.server.service.data.provider;

import com.starbuds.server.common.constant.data.DataGreenTypeEnum;
import com.starbuds.server.common.constant.data.GreenBizTypeEnum;
import com.starbuds.server.common.constant.data.GreenCompanyTypeEnum;
import com.starbuds.server.common.pojo.api.CallApiTrace;
import com.starbuds.server.common.pojo.data.RiskParms;
import com.starbuds.server.common.pojo.data.green.GreenAuditTrace;
import com.starbuds.server.common.pojo.db.DataGreenHisDO;
import com.starbuds.server.common.service.data.DataGreenHisService;
import com.starbuds.server.service.data.entity.green.GreenRobotAuditResult;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.beans.BeanUtils;

import javax.annotation.Resource;
import java.util.function.Function;
import java.util.function.Supplier;

/*xxxAudit如loginAudit audit ->loginCensor风控实现类请求数据 -> 返回数据 设备指纹、风向，激活注册登录等业务风控*/
public abstract class RiskProvider {

    @Resource
    private DataGreenHisService dataGreenHisService;

    /**
     * 请求第三方数据
     *
     * @param json
     * @return
     */
    protected abstract CallApiTrace loginCensor(RiskParms json);

    protected abstract CallApiTrace registCensor(RiskParms json);

    protected abstract CallApiTrace activationCensor(RiskParms json);

    /**
     * 获得审核结果
     *
     * @param rsp 图片审核返回数据
     * @return
     */
    protected abstract GreenRobotAuditResult loginResult(String rsp);

    protected abstract GreenRobotAuditResult registResult(String rsp);

    protected abstract GreenRobotAuditResult activationResult(String rsp);


    /**
     * 登录用户审核
     *
     * @return
     */
    public GreenAuditTrace loginAudit(Long bizId, RiskParms parms) {
        return audit(bizId, GreenBizTypeEnum.USER_LOGIN, DataGreenTypeEnum.TEXT, () -> loginCensor(parms), (res) -> loginResult(res));
    }


    /**
     * 注册用户审核
     *
     * @return
     */
    public GreenAuditTrace registAudit(Long bizId, RiskParms parms) {
        return audit(bizId, GreenBizTypeEnum.USER_REGIST, DataGreenTypeEnum.TEXT, () -> registCensor(parms), (res) -> registResult(res));
    }


    /**
     * 激活用户审核
     *
     * @return
     */
    public GreenAuditTrace activationAudit(Long bizId, RiskParms parms) {
        //CallApiTrace apiTrace = activationCensor(parms);
        //GreenRobotAuditResult robotAuditResult = activationResult(apiTrace.getRspData());
        return audit(bizId, GreenBizTypeEnum.USER_ACTIVATION, DataGreenTypeEnum.TEXT, () -> activationCensor(parms), (res) -> activationResult(res));
    }


    /**
     * 内容审核公司类型
     *
     * @return
     */
    protected abstract GreenCompanyTypeEnum companyType();


    /**
     * @param callApiTraceSupplier     请求接口解析
     * @param robotAuditResultFunction 审核结果解析
     * @return , () -> activationCensor(parms), (res) -> activationResult(res)
     */
    private GreenAuditTrace audit(Long bizId, GreenBizTypeEnum bizType, DataGreenTypeEnum dataGreenType, Supplier<CallApiTrace> callApiTraceSupplier, Function<String, GreenRobotAuditResult> robotAuditResultFunction) {

        GreenAuditTrace trace = new GreenAuditTrace();
        trace.setBizId(bizId);
        trace.setBizType(bizType);
        trace.setDataType(dataGreenType);

        // 计算API耗时
        long startTime = System.currentTimeMillis();
        // 调用get方法时，会触发"() -> activationCensor(parms)"方法调用，请求数据得到结果对象callApiTrace
        CallApiTrace callApiTrace = callApiTraceSupplier.get();
        long spendTime = System.currentTimeMillis() - startTime;
        BeanUtils.copyProperties(callApiTrace, trace);
        trace.setSpendTime(NumberUtils.toInt(spendTime + ""));

        // 解析审核结果
        // 调用apply方法传入string数据，会触发"(res) -> activationResult(res))"方法调用，解析数据得到对象robotAuditResult
        GreenRobotAuditResult robotAuditResult = robotAuditResultFunction.apply(trace.getRspData());
        trace.setRobotAuditStatus(robotAuditResult.getRobotAuditStatus());
        BeanUtils.copyProperties(robotAuditResult, trace);

        DataGreenHisDO po = new DataGreenHisDO();
        BeanUtils.copyProperties(trace, po);
        po.setBizType(trace.getBizType().getCode());
        po.setDataType(trace.getDataType().getCode());
        po.setRobotAuditStatus(trace.getRobotAuditStatus().value());
        GreenCompanyTypeEnum companyType = companyType();
        companyType = companyType == null ? GreenCompanyTypeEnum.UNKNOWN : companyType;
        po.setCompanyType(companyType.getCode());

        dataGreenHisService.create(po);

        return trace;
    }

}
