package com.starbuds.server.service.data.redis.listener;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.mq.QueueRtcGiftComboTimeoutMsg;
import com.starbuds.server.service.data.redis.RedisUtil;
import com.starbuds.server.service.data.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.connection.Message;
import org.springframework.data.redis.connection.MessageListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/4/5
 */
@Component
@Slf4j
public class RedisKeyExpireMsgListener implements MessageListener {

    @Resource
    RedisUtil redisUtil;

    @Resource
    RocketProducer rocketProducer;

    /**
     * Callback for processing received objects through Redis.
     *
     * @param message message must not be {@literal null}.
     * @param pattern pattern matching the channel (if specified) - can be {@literal null}.
     */
    @Override
    public void onMessage(Message message, byte[] pattern) {
        log.info(JSON.toJSONString(message, true));
        String key = new String(message.getBody());
        if (key.contains(RedisRtcKeys.RTC_GIFT_COMBO)) {
            String comboKey = key.replace(RedisRtcKeys.RTC_GIFT_COMBO, "");
            Long timestamp = Optional.ofNullable(redisUtil.hGet(RedisRtcKeys.RTC_COMBO_TIME, comboKey)).map(Convert::toLong).orElse(null);
            if (timestamp == null) {
                return;
            }

            String hisKey = RedisRtcKeys.getRtcComboHis(key, timestamp);
            Long combo = Convert.toLong(redisUtil.getAndSet(hisKey, "-1"), -1L);
            if (combo < 0) {
                return;
            }

            String targetsKey = RedisRtcKeys.getRtcComboTargets(key, timestamp);
            String targets = redisUtil.get(targetsKey);
            List<Long> targetUserIds = Arrays.stream(targets.split(",")).map(Convert::toLong).collect(Collectors.toList());

            // 清理资源
            redisUtil.hDelete(RedisRtcKeys.RTC_COMBO_TIME, comboKey);
            redisUtil.delete(Arrays.asList(key, hisKey, targetsKey));

            // 解析 key 房间id:用户id:礼物id:数量
            String[] params = comboKey.split("_");
            // 丢出礼物连击过期消息
            QueueRtcGiftComboTimeoutMsg msg = new QueueRtcGiftComboTimeoutMsg();
            msg.setRoomId(Convert.toLong(params[0]));
            msg.setSenderUserId(Convert.toLong(params[1]));
            msg.setGiftId(Convert.toLong(params[2]));
            msg.setQuantity(Convert.toInt(params[3]));
            msg.setCombo(combo);
            msg.setTargetIds(targetUserIds);
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.RTC_GIFT_COMBO_TIMEOUT, msg.serialize());
        }
    }
}
