package com.starbuds.server.service.data.util;

import okhttp3.*;

import java.io.IOException;
import java.util.Map;

/**
 * @author aoe
 * @date 2021/1/7
 */
public class HttpUtil {

    private static OkHttpClient client;

    private static final MediaType MEDIA_TYPE_JSON = MediaType.get("application/json; charset=utf-8");

    static {
        client = new OkHttpClient();
    }

    private HttpUtil() {
    }

    public static String post(String url, String json) throws IOException {
        RequestBody body = RequestBody.create(json, MEDIA_TYPE_JSON);
        Request request = new Request.Builder()
                .url(url)
                .post(body)
                .build();
        try (Response response = client.newCall(request).execute()) {
            return response.body().string();
        }
    }

    public static String post(String url, String json, Map<String, String> headers) throws IOException {
        RequestBody body = RequestBody.create(json, MEDIA_TYPE_JSON);

        Request request = new Request.Builder()
                .url(url)
                .post(body)
                .headers(Headers.of(headers))
                .build();
        try (Response response = client.newCall(request).execute()) {
            return response.body().string();
        }
    }

    public static String postForm(String url, Map<String, String> params) throws IOException {
        Request.Builder builder = new Request.Builder();
        if (params != null) {
            FormBody.Builder body = new FormBody.Builder();
            for (Map.Entry<String, String> param : params.entrySet()) {
                body.add(param.getKey(), param.getValue());
            }
            builder.post(body.build());
        }

        Request request = builder.url(url).build();

        try (Response response = client.newCall(request).execute()) {
            return response.body().string();
        }
    }

    public static String get(String url) throws IOException {
        Request request = new Request.Builder()
                .url(url)
                .get()
                .build();
        try (Response response = client.newCall(request).execute()) {
            return response.body().string();
        }
    }
}
