package com.starbuds.server.service.data.impl;

import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.constant.message.sms.SmsActionEnum;
import com.starbuds.server.common.pojo.data.IpLocation;
import com.starbuds.server.common.service.data.AreaService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.test.BaseTest;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.testng.annotations.Test;

import java.util.concurrent.TimeUnit;

import static org.testng.Assert.*;

@Slf4j
public class AreaServiceImplTest extends BaseTest {

    @Reference(version = "1.0.0")
    private AreaService areaService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Test
    public void testGetIpLocation() {
        IpLocation ipLocation = areaService.getIpLocation("122.234.10.244");
        log.info("ipLocation = {}", LogUtil.obj2JsonObj(ipLocation));
    }

    @Test
    public void testRedis(){
        String phone = "18667166831";
        //设置过期时间
        redisService.set(getRedisKey(SmsActionEnum.LOGIN, phone), "4525", 60*1000, TimeUnit.MILLISECONDS);

        //剩余生存时间(秒)
        long ttl = redisService.getTtl(getRedisKey(SmsActionEnum.LOGIN, phone));
        log.info("剩余秒="+ttl+"");
    }

    private String getRedisKey(SmsActionEnum action, String phone) {
        String redisKey = "";
        switch (action) {
            case LOGIN:
                redisKey = RedisCacheKeys.getSmsLoginKey(phone);
                break;
            case REGISTER:
                redisKey = RedisCacheKeys.getSmsRegKey(phone);
                break;
            case FORGET_PASSWORD:
                redisKey = RedisCacheKeys.getSmsForget(phone);
                break;
            case BIND:
                redisKey = RedisCacheKeys.getSmsBind(phone);
                break;
            default:
                break;
        }
        return redisKey;
    }

}