package com.starbuds.server.service.data.provider;

import com.starbuds.server.common.constant.data.DataGreenTypeEnum;
import com.starbuds.server.common.constant.data.GreenAuditStatusEnum;
import com.starbuds.server.common.constant.data.GreenBizTypeEnum;
import com.starbuds.server.common.pojo.data.RiskParms;
import com.starbuds.server.common.pojo.data.green.GreenAuditTrace;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.test.BaseTest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomUtils;
import org.junit.Assert;
import org.testng.annotations.Test;

import javax.annotation.Resource;

@Slf4j
public class AbstractGreenProviderTest extends BaseTest {

    @Resource(name = "baiduGreenProvider")
    private AbstractGreenProvider baiduGreenProvider;


    @Resource(name = "shumeiRiskProvider")
    private RiskProvider shumeiRiskProvider;

    @Test
    public void imgAudit() {
        // 二维码
        String image = "http://feed.qn.panygo.com/image/341127609712640/1615795890440.png";
        GreenAuditTrace trace = baiduGreenProvider.imgAudit(image, "12345", RandomUtils.nextLong(), GreenBizTypeEnum.USER_AVATAR, DataGreenTypeEnum.IMAGE);
        log.info("Abstract-百度图片审核 {}", LogUtil.obj2JsonObj(trace));
        Assert.assertEquals(GreenAuditStatusEnum.BLOCK_ROBOT, trace.getRobotAuditStatus());

        image = "http://feed.qn.panygo.com/image/160835024453632/1628673420203.png";
        trace = baiduGreenProvider.imgAudit(image, "12345", RandomUtils.nextLong(), GreenBizTypeEnum.USER_AVATAR, DataGreenTypeEnum.IMAGE);
        log.info("Abstract-百度图片审核 {}", LogUtil.obj2JsonObj(trace));
        Assert.assertEquals(GreenAuditStatusEnum.PASSED_ROBOT, trace.getRobotAuditStatus());
    }

    @Test
    public void textAudit() {
        String text = "约炮";
        GreenAuditTrace trace = baiduGreenProvider.textAudit(text, "12345", RandomUtils.nextLong(), GreenBizTypeEnum.USER_NAME, DataGreenTypeEnum.TEXT);
        log.info("Abstract-百度文本审核 {}", LogUtil.obj2JsonObj(trace));
        Assert.assertEquals(GreenAuditStatusEnum.BLOCK_ROBOT, trace.getRobotAuditStatus());

        text = "阳光";
        trace = baiduGreenProvider.textAudit(text, "12345", RandomUtils.nextLong(), GreenBizTypeEnum.USER_NAME, DataGreenTypeEnum.TEXT);
        log.info("Abstract-百度文本审核 {}", LogUtil.obj2JsonObj(trace));
        Assert.assertEquals(GreenAuditStatusEnum.PASSED_ROBOT, trace.getRobotAuditStatus());
    }

    @Test
    public void textActivation() {
        RiskParms p = new RiskParms();
        p.setIp("115.199.106.44");
        p.setUserID(null);
        p.setTimestamp(System.currentTimeMillis());
        p.setDeviceId("testaaa");
        p.setOs("android");
        p.setAppVersion("1.7.0");
        p.setAdvertisingId(RandomUtils.nextLong() + "");
        p.setApputm("test");
        p.setLat(null);
        p.setLng(null);
        GreenAuditTrace trace = shumeiRiskProvider.activationAudit(RandomUtils.nextLong(), p);

        log.info("Abstract-数美审核 {}", LogUtil.obj2JsonObj(trace));

    }

    @Test
    public void textLogin() {

        //{"eventId":"login","data":{"isTokenSeperate":1,"appVersion":"1.6.5","gpsLocation":{"gpsLongitude":120.074379,"gpslatitude":30.292166},"tokenId":"9294409891907","os":"iOS","ip":"115.199.102.152","deviceId":"BFSkPBDBqpaonXHB7d09Lg72dqxqY8WlQytSDJvH9Dl7JhKZAE2YFqBJIU0cBEi0+8EMHaavK1gMOlnuGsCMJYg==","timestamp":1630563347291},"accessKey":"6E5S9Sz3rvB67Y73RKQh","appId":"default"}

        RiskParms p = new RiskParms();
        p.setIp("192.168.1.1");
        p.setUserID(RandomUtils.nextLong());
        p.setTimestamp(System.currentTimeMillis());
        p.setDeviceId("testaaa");
        p.setOs("android");
        p.setAppVersion("1.7.0");
        p.setLat(null);
        p.setLng(null);
        GreenAuditTrace trace = shumeiRiskProvider.loginAudit(RandomUtils.nextLong(), p);

        log.info("Abstract-数美审核 {}", LogUtil.obj2JsonObj(trace));

    }

    @Test
    public void textReigster() {
        RiskParms p = new RiskParms();
        p.setIp("192.168.1.1");
        p.setUserID(RandomUtils.nextLong());
        p.setTimestamp(System.currentTimeMillis());
        p.setDeviceId("testaaa");
        p.setOs("android");
        p.setAppVersion("1.7.0");
        p.setLat(null);
        p.setLng(null);
        GreenAuditTrace trace = shumeiRiskProvider.registAudit(RandomUtils.nextLong(), p);

        log.info("Abstract-数美审核 {}", LogUtil.obj2JsonObj(trace));

    }

}